/*********************************************************/
/* utilities for decoding data from the chips            */
/* and to check that the data format is correct          */
/*********************************************************/
/* charge injection capacitor in femtofarad       */
#define CHARGE_INJ_CAPACITOR  65

/* calibration dac step size in Volts             */
#define CAL_DAC_STEP 0.008217

/* typical channel to use in case a channel is bad */  
#define TYP_CHAN_NUM 0

#define TRUE               1
#define FALSE              0


#include "cal_svtdef.h"
#include "tk.h"
#include <stdio.h>
#include <math.h>
#include "error.h"
#include "reglist.h"
#include "cal_functions.h"
#include "cal_format_check.h"
#include "cal_global.h"

int Gc_to_bin(int);

/*---------------------------------------------------------------*/
/* Check of data format.                                         */
/* Must be called on event record.                               */
/* Returns "OK" or string with error message.                    */
/* flag is a bitmask of the checks to be performed.              */
/*---------------------------------------------------------------*/
char* cal_format_check(char* pointer, int flag) {

  int nchips, nbytes, dummy, i, j, t1, t2, ishift, nb, header_check, temp;
  static char return_code[200];
  int begin_data[MAX_NUM_CHIP], end_data[MAX_NUM_CHIP];
  int hit_map[MAX_NUM_CHIP][4];
  int chnum, tot, tstamp;
  int ev_trcount, ev_trtstamp, last_address; 
  int startbit, address, event_data, trcount, trtstamp;
  int hit_check, bit;

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
/* extract number of bytes                                        */
/* min num of bytes is 12 + (nbytes-runnum-evnum)                 */
/*                      1 + (num chips)                           */
/*                      4 + (1 chip, CR)                          */
/*                     32 + (1 chip, MSK)                         */
/*                      8 + (1 chip, begin/end)                   */
/*                      4 = (1 chip, no hits)                     */
/*                     61                                         */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
  dummy  = cal_extract(1,&nbytes,pointer);
  if (dummy == 1) {
    sprintf(return_code,"Error reading num bytes from event record");
    return return_code;
  }
  if (nbytes < 61) {
    sprintf(return_code,"Only %d bytes in event record",nbytes);
    return return_code;
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
/* get number of chips                                            */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
  nchips = (int) *(pointer+12);
  nchips = nchips & 0xff;
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
/* Get data pointers for chips                                    */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
  ishift = 12 + 1 + 4*nchips + 32*nchips;
  for (i=0; i<nchips; i++) {
    dummy = cal_extract(1,&temp,(pointer+ishift));
    begin_data[i] = temp;
    ishift = ishift + 4;
  }
  for (i=0; i<nchips; i++) {
    dummy = cal_extract(1,&temp,(pointer+ishift));
    end_data[i] = temp;
    ishift = ishift + 4;
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
/* checks of number of bytes                                      */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
  if ( (flag & NUMBER_OF_BYTES) != 0) { 
    for (i=0; i<nchips; i++) {
      nb = end_data[i] - begin_data[i] + 1;
      if (nb < 4) {
	sprintf(return_code,"chip %d has only %d bytes",i,nb);
        return return_code;
      }
      if (nb%2 != 0) {
	sprintf(return_code,"chip %d has odd no. of bytes= %d",i,nb);
	return return_code;
      }
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
/* check of trailer                                               */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
  if ( (flag & TRAILER) != 0) {
    for (i=0; i<nchips; i++) {
      t1 = (int) *(pointer+end_data[i]-1);
      t1 = t1 & 0xff;
      t2 = (int) *(pointer+end_data[i]);
      t2 = t2 & 0xff;
      if (t1 == 0x80) t1=0x0;           /* glitch on trailer allowed */
      if ( t1 != 0 || t2 != 0) {
	sprintf(return_code,"bad trailer on chip %d",i);
	return return_code;
      }
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
/* check of headers                                               */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
  header_check = START_BIT || EVENT_DATA_STATUS || TRIGGER_TAG ||
                 CHIP_ADDRESS;
  if ( (flag & header_check) != 0) {
    for (i=0; i<nchips; i++) {
      ishift = begin_data[i];
      cal_get_header(pointer, ishift, &startbit, &address, &event_data, 
                   &trcount, &trtstamp);
      if ( (flag & START_BIT) != 0 && startbit == 0 && ignore[i] == 0) {
	sprintf(return_code,"chip %d misses startbit",i);
	return return_code;
      }
      if ( (flag & EVENT_DATA_STATUS) != 0 && event_data == 0 && 
	                                                  ignore[i] == 0) {
	sprintf(return_code,"chip %d no event-data bit",i);
	return return_code;
      }
      if (i == 0) {
	ev_trcount  = trcount;
	ev_trtstamp = trtstamp;
      } else if ( (flag & TRIGGER_TAG) != 0 && trcount != ev_trcount &&
                                             ignore[i] == 0) {
	sprintf(return_code,"chip 0 trig tag %d - chip %d trig tag %d",
		ev_trcount,i,trcount);
	return return_code;
      } else if ( (flag & TRIGGER_TIME) != 0 && trtstamp != ev_trtstamp &&
		                             ignore[i] == 0) {
	sprintf(return_code,"chip 0 trig time %d - chip %d trig time %d",
		ev_trtstamp,i,trtstamp);
	return return_code;
      }
      if ( (flag & CHIP_ADDRESS) != 0) {
	if (i==0 && address != 0) {
	  sprintf(return_code,"Invalid address = %d on chip %d",address,i);
	  return return_code;
	} else if (i==0) {
	  last_address = 0;
	} else if ( address != (last_address+1) && address != 0) {
	  sprintf(return_code,"Invalid address = %d on chip %d",address,i);
	  return return_code;
	} else {
	  last_address = address;
	}
      }
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
/* Check of hits                                                  */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
  hit_check = VALID_CHANNEL_NUMBER || NO_REPEAT_CHANNEL || NON_ZERO_TOT;
  if ( (flag & hit_check) != 0) {
    for (i=0; i<nchips; i++) {
      if (ignore[i] == 1) continue;
      if ( (flag & NO_REPEAT_CHANNEL) != 0) {
	for (j=0; j<4; j++) hit_map[i][j] = 0;
      }
      for (ishift=(begin_data[i]+2); ishift<(end_data[i]-2); ishift=ishift+2){ 
	cal_get_hit(pointer, ishift, &chnum, &tot, &tstamp);
	if ( (flag & NON_ZERO_TOT) != 0 && tot == 0) {
	  sprintf(return_code,"Hit with TOT=0 on Channel=%d Chip=%d",chnum,i);
	  return return_code;
	}
	if ( (flag & VALID_CHANNEL_NUMBER) != 0 && chnum > 127) {
	  sprintf(return_code,"Illegal channel num=%d on chip=%d",chnum,i);
	  return return_code;
	}
	if ( (flag & NO_REPEAT_CHANNEL) != 0) {
	  j   = chnum/32;
	  bit  = chnum - 32*j;
	  temp = 1;
	  if (bit != 0) temp = temp << bit;
	  if ( (hit_map[i][j] & temp) != 0) {
	    sprintf(return_code,"More than one hit on channel %d chip %d",
		    chnum,i);
	    return return_code;
	  }
	  hit_map[i][j] = hit_map[i][j] | temp;
	}
      }
    }
  }
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
/* done                                                           */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  -*/
  sprintf(return_code,"OK");
  return return_code;
}



/*---------------------------------------------------------------*/
/* Unpack hit                                                    */
/*---------------------------------------------------------------*/
void cal_get_hit(char* pointer, int ishift, int *chnum, 
int *tot, int *tstamp)
{
  int hit, temp;
  hit    = 0;
  hit    = (int) *(pointer + ishift);
  hit    = hit & 0xff;
  hit    = hit << 8;
  temp   = 0;
  temp   = (int) *(pointer + ishift + 1);
  temp    = temp & 0xff;      
  hit    = hit | temp;
  *chnum  = (hit >> 9) & 0x7f;
  *chnum  = switch_bits(*chnum,7);
  *tot    = hit & 0xf;
  *tot    = switch_bits(*tot,4);
  *tot    = Gc_to_bin(*tot);
  *tstamp = hit & 0x1f0;
  *tstamp = *tstamp >> 4;
  *tstamp = switch_bits(*tstamp,5);
  *tstamp = Gc_to_bin(*tstamp);
  return;
}
/*--------------------------------------------------------------------*/
/* Unpack header                                                      */
/*--------------------------------------------------------------------*/
void cal_get_header(char* pointer, int jshift, int *startbit, 
		    int *address, 
                    int *event_data, int *trcount, int *trtstamp)

{
  int header, temp;
  header  = 0;
  header  = (int) *(pointer + jshift);
  header  = header & 0xff;
  header  = header << 8;
  temp    = 0;
  temp    = (int) *(pointer + jshift + 1);
  temp    = temp & 0xff;
  header  = header | temp;
  *startbit   = header & 0x8000;
  *startbit   = *startbit >> 15;
  *address    = header & 0x7800;
  *address    = *address >> 11;
  *address    = switch_bits(*address,4);
  *event_data = header & 0x400;
  *event_data = *event_data >> 10;
  *trcount    = header & 0x3e0;
  *trcount    = *trcount >> 5;
  *trcount    = switch_bits(*trcount,5);
  *trtstamp   = header & 0x1f;
  *trtstamp   = switch_bits(*trtstamp,5);
  return;
}
/*--------------------------------------------------------------------*/
/*  Gray code to binary conversion                                    */
/*--------------------------------------------------------------------*/
int Gc_to_bin(a)
int a;
{   
   switch(a){
      case 0: return(0);
      case 1: return(1);
      case 3: return(2);
      case 2: return(3);
      case 6: return(4);
      case 7: return(5);
      case 5: return(6);
      case 4: return(7);
      case 12: return(8);
      case 13: return(9);
      case 15: return(10);
      case 14: return(11);
      case 10: return(12);
      case 11: return(13);
      case 9: return(14);
      case 8: return(15);
      case 24: return(16);
      case 25: return(17);
      case 27: return(18);
      case 26: return(19);
      case 30: return(20);
      case 31: return(21);
      case 29: return(22);
      case 28: return(23);
      case 20: return(24);
      case 21: return(25);
      case 23: return(26);
      case 22: return(27);
      case 18: return(28);
      case 19: return(29);
      case 17: return(30);
      case 16: return(31);
      default : return(-1);
    }
}
/**************************************************************************/
/*                                                                        */ 
/* Function to take the Time-Over-Threshhold output for a given channel   */
/*   and convert this to a charge (units of fC).  Channel parameters      */
/*   are stored in the database file "3x3parms.dat", which has been       */
/*   created with OUTDATA.KUMAC (from output of TOTFITS.KUMAC).           */
/*                                                                        */
/**************************************************************************/
char* cal_tot_convert( int tot, int chip, int chan, float* charge ) {


  float tmp1, Cal_Dac;
  
  int i,j;
  int max_line = 80;
  int Tot_Extrap = 32;           /* Value suggested by Claudio */

  char *Data_File = "/homes/babar/oasis_new/3x3parms.dat";
  char line[80];
  static char return_code[200];
  static int ReadParmsFile = FALSE;
  static float Channel_Parms[128*MAX_NUM_CHIP][5];

  FILE *fp;


/*--------------------------------------------------------------------*/
/* if it is not the 3x3 return one                                    */
/*--------------------------------------------------------------------*/
  if (number_of_chips != 6) {
     *charge = 1.0;
     sprintf(return_code, "OK");  
     return return_code;
   }
/*--------------------------------------------------------------------*/
/* Retrieve Parameters for this Channel, first run only.... */
/*--------------------------------------------------------------------*/

  if (ReadParmsFile == FALSE) {
/*--------------------------------------------------------------------*/
/* Open Database File, read                                           */
/* Channel Parameters into array                                      */
/* Check to make sure Data File exists....                            */
/*--------------------------------------------------------------------*/
    if ((fp = fopen(Data_File, "r")) == NULL) {
      sprintf(return_code, "cal_tot_convert: can't open datafile %s\n",
	      *Data_File);
      *charge = 0;
      return return_code;
    }
    for ( i=0 ; i < 128*number_of_chips ; i++ ) {

      if (fgets ( line, max_line, fp ) == NULL) { 
 	sprintf(return_code, "cal_tot_convert: premature end of" 
 		"parameter file %s.", *Data_File); 
	fclose(fp);                    /* Close Database File */ 
	return return_code; 
      } 
      sscanf (line, "%f %f %f %f %f", &Channel_Parms[i][1],  
	      &Channel_Parms[i][2], &Channel_Parms[i][3], 
	      &Channel_Parms[i][4], &Channel_Parms[i][5] );

/*  Echo input to screen......                                           */
/*  printf("%f %f %f %f %f\n", Channel_Parms[i][1], Channel_Parms[i][2], */ 
/*  Channel_Parms[i][3], Channel_Parms[i][4], Channel_Parms[i][5] );     */
     }
    
    fclose(fp);                /* Close Database File */
    ReadParmsFile = TRUE;
  }

/* Make sure Chip # and Channel # are Within Bounds */

  if ( chip > number_of_chips-1 || chip < 0 ) {
    sprintf(return_code, "cal_tot_convert: Chip Number %i Out of"
	    " Bounds.", chip);
    *charge = 0;
    return return_code;
  }

  if ( chan > 127 || chan < 0 ) {
    sprintf(return_code, "cal_tot_convert: Channel Number %i Out of"
	    " Bounds.", chan);
    *charge = 0;
    return return_code;
  }

/* Compute Cal_Dac from input TOT */


  i = chip*128 + chan;

  for (j = 0 ; j < 128; j++) {       /* If chosen channel has NULL parms,    */
    if (Channel_Parms[i][1] > 0)     /*  then check Ch TYP_CHAN_NUM on this  */
      break;                         /*  on this chip, or failing this, next */
                                     /*  10 channels.                        */
    i = chip*128 + TYP_CHAN_NUM + j; /* Currently, TYP_CHAN_NUM = 0          */
  }

  if (Channel_Parms[i][1] == 0) {    /* If STILL have Null parms... */
    Cal_Dac = 0;                     /*   ...curse, set charge = 0  */
    *charge = 0;
  }
  else if (tot >= Channel_Parms[i][1]) {
    tmp1 = Channel_Parms[i][3]*pow(Tot_Extrap, Channel_Parms[i][4]);
    Cal_Dac = 1/Channel_Parms[i][5]*(tot-Channel_Parms[i][1] + 
				     Tot_Extrap*Channel_Parms[i][5] + 
				     Channel_Parms[i][2]*exp(-tmp1));
    *charge = Cal_Dac * CHARGE_INJ_CAPACITOR * CAL_DAC_STEP;
  }
  else {
    tmp1 = log( (Channel_Parms[i][1]-tot)/Channel_Parms[i][2] );
    Cal_Dac = pow( -tmp1/Channel_Parms[i][3], 1/Channel_Parms[i][4] );

                            /* Check to see if Cal_Dac is above Tot_Extrap */
    if (Cal_Dac >= Tot_Extrap) {    /* and if so, Extrapolate.....         */
      tmp1 = Channel_Parms[i][3]*pow(Tot_Extrap, Channel_Parms[i][4]);
      Cal_Dac = 1/Channel_Parms[i][5]*(tot-Channel_Parms[i][1] + 
				      Tot_Extrap*Channel_Parms[i][5] + 
				      Channel_Parms[i][2]*exp(-tmp1));
    }
    *charge = Cal_Dac * CHARGE_INJ_CAPACITOR * CAL_DAC_STEP;

  }

  if (i == 128*chip+chan)          /* Return "OK" if chosen channel good */
    sprintf(return_code, "OK");    /* and an error if its not.           */
  else 
    sprintf(return_code, "CHANNEL DEAD.  USED 1st AVAILABLE CHANNEL");
    return return_code;

}



