/************************************************************************/
/* This file contains all of the functions needed for the XY stage.     */
/*                                                                      */
/* All of the functions are of type char*, and they are expected        */
/* to return "OK" if no errors were encountered.  In case of error of   */
/* any kind, return an error message.  This error message will be       */
/* displayed for you, and the run will be aborted.                      */
/*                                                                      */
/* Even if your setup does not have any XY stage capabilities, you      */
/* still need to compile this code.  In oasis_new/src/cal/site.h        */
/* you should have a line with                                          */
/*              #define YOURSITENAME                                    */
/* Look at that file and change to YOURSITENAME if necessary.           */
/* Make sure that one and only one site is defined in this header file  */
/* The code that gets compiled is all of the code between               */
/*              #ifdef YOURSITENAME                                     */
/* and                                                                  */
/*              #endif                                                  */
/* If your site does not have an XY stage, you need to dummy-out all    */
/* of the functions.  See the snippet of code for the LBL site. Add     */
/* an identical piece of code between #ifdef YOURSITENAME and #endif.   */
/* On the other hand, if you want to do XY scans, you'll have to write  */
/* your own code.  Once you are satisfied with the code you wrote,      */
/* send it to claudio@ucsb2.physics.ucsb.edu.  This way it will be      */
/* included in the next release and you will not have to go through     */
/* this process again.                                                  */
/*                                                                      */
/* Now for a description of the functions.  "axis" is the axis number,  */
/* "coord" is the coordinate. The meaning of coordinate is up to you.   */
/* Here it goes:                                                        */
/*                                                                      */
/* char* cal_xy_init_axis(Tcl_Interp* interp, int axis)                 */
/*                                  do any kind of initialization that  */
/*                                  may be necessary before starting    */
/*                                  the scan.                           */
/*                                                                      */
/* char* cal_xy_move_to(Tcl_Interp* interp, int axis, float coord)      */
/*                                  move axis to coord                  */
/*                                                                      */
/* char* cal_xy_get_coord(Tcl_Interp* interp, int axis, float* coord)   */
/*                                  returns the position (coord) of     */
/*                                  axis.  How this is done is entirely */
/*                                  up to you.  It is important because */
/*                                  the value of coord is then written  */
/*                                  out in the data stream, and is used */
/*                                  in histogram filling.               */
/*                                                                      */
/* char* cal_xy_end_axis(Tcl_Interp* interp, int axis)                  */
/*                                  call at the end of the scan         */
/*                                                                      */
/************************************************************************/
#include "cal_svtdef.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "error.h"
#include "reglist.h"
#include "tk.h"
#include "cal_global.h"
#include "cal_functions.h"
#include "site.h"

/************************************************************************/
/* Here is the LBL code.  No XY axes there                              */
/************************************************************************/
#if defined(LBL) || defined(SUNBABAR) || defined(FERMI2) || defined(SLAC)
char* cal_xy_init_axis(Tcl_Interp* interp, int axis) {
  static char return_message[] = "No XY scans implemented at LBL";
  return return_message;
}
char* cal_xy_end_axis(Tcl_Interp* interp, int axis) {
  static char return_message[] = "No XY scans implemented at LBL";
  return return_message;
}
char* cal_xy_get_coord(Tcl_Interp* interp, int axis, float* coord) {
  static char return_message[] = "No XY scans implemented at LBL";
  return return_message;
}
char* cal_xy_move_to(Tcl_Interp* interp, int axis, float coord) {
  static char return_message[] = "No XY scans implemented at LBL";
  return return_message;
}
#endif
/************************************************************************/
/* End of the LBL code                                                  */
/************************************************************************/

/************************************************************************/
/* Here is the UCSB code                                                */
/************************************************************************/
#ifdef UCSB
/*- - - - - - - - - - - - - - - - - - -  -  - - - - - - - - */
/* No initializations necessary                             */
/*- - - - - - - - - - - - - - - - - - -  -  - - - - - - - - */
char* cal_xy_init_axis(Tcl_Interp* interp, int axis) {
  static char return_message[] = "OK";
  return return_message;
}
/*- - - - - - - - - - - - - - - - - - -  -  - - - - - - - - */
/* Nothing needed at the end                                */
/*- - - - - - - - - - - - - - - - - - -  -  - - - - - - - - */
char* cal_xy_end_axis(Tcl_Interp* interp, int axis) {
  static char return_message[] = "OK";
  return return_message;
}
/*- - - - - - - - - - - - - - - - - - -  -  - - - - - - - - */
/* move to a position                                       */
/*- - - - - - - - - - - - - - - - - - -  -  - - - - - - - - */
char* cal_xy_move_to(Tcl_Interp* interp, int axis, float coord) {
static char return_message[200];
static char cmd[200];
int dummy;

  if (axis == 1) {
     sprintf(cmd,"move_to_position X %f",coord);
   } else if (axis == 2) {
     sprintf(cmd,"move_to_position Y %f",coord);
   } else {
    sprintf(return_message,"Call to cal_xy_move_to with illegal axis no. %d",
            axis);
    return return_message;
  }
  dummy = Tcl_Eval(interp,cmd);
  return interp->result;
}
/*- - - - - - - - - - - - - - - - - - -  -  - - - - - - - - */
/* get the coordinate                                       */
/*- - - - - - - - - - - - - - - - - - -  -  - - - - - - - - */
char* cal_xy_get_coord(Tcl_Interp* interp, int axis, float* coord) {
static char return_message[200];
static char cmd[200];
int dummy;

  if (axis == 1) {
    sprintf(cmd,"find_x_or_y X 1200");
  } else if (axis == 2) {
    sprintf(cmd,"find_x_or_y Y 1200");
  } else {
    sprintf(return_message,"Call to cal_xy_get_coord with illegal axis no. %d",
            axis);
    return return_message;
  }
  dummy = Tcl_Eval(interp,cmd);
  if (strcmp(interp->result,"999") == 0) {
    strcpy(return_message,"Error reading coordinate");
    return return_message;
  }
  sscanf(interp->result,"%f",coord);
  strcpy(return_message,"OK");
}
#endif
/************************************************************************/
/* End of the UCSB code                                                 */
/************************************************************************/

