/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 * modified to include motor control functions..
 *    Sanjay Hortikar (horti@physics.ucsb.edu)
 */

#include <tk.h>
#include <stdlib.h>
#include "cwish.h"
#include "reglist.h"
#include "cardlist.h"
#include "error.h"
#include "front.h"
#include "lock.h"
#include "tclutils.h"
#include "regtrace.h"
#include "rpc.h"

#ifndef FRONTEND
#include "rpc_client.h"
#endif

/* 
 * new tcl commands
 *
 */


static int vme_WriteCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_register reg;
  int value;
  int inst;
  vme_rc rc;
  if (argc != 3) return vme_TclArgError(interp,*argv,3);
  rc = vme_TclGetRegister(interp,*++argv,&reg,&inst); 
  if (vmeError(rc)) return vme_TclReturn(interp,rc); 
  if (Tcl_GetInt(interp,*++argv,&value) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  return vme_TclReturn(interp,vmeWrite(inst,reg,(vme_vmetype)value));
}

static int vme_ReadCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_register reg;
  vme_rc rc;
  int inst;
  if (argc != 2) return vme_TclArgError(interp,*argv,2);
  rc = vme_TclGetRegister(interp,*++argv,&reg,&inst); 
  if (vmeError(rc)) return vme_TclReturn(interp,rc); 
  return vme_TclRcReturn(interp,vmeRead(inst,reg));
}

static int vme_SetCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_register reg;
  vme_rc rc;
  int inst;
  if (argc != 2) return vme_TclArgError(interp,*argv,2);
  rc = vme_TclGetRegister(interp,*++argv,&reg,&inst); 
  if (vmeError(rc)) return vme_TclReturn(interp,rc); 
  return vme_TclReturn(interp,vmeSet(inst,reg));
}

static int vme_ClearCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_register reg;
  vme_rc rc;
  int inst;
  if (argc != 2) return vme_TclArgError(interp,*argv,2);
  rc = vme_TclGetRegister(interp,*++argv,&reg,&inst); 
  if (vmeError(rc)) return vme_TclReturn(interp,rc); 
  return vme_TclReturn(interp,vmeClear(inst,reg));
}

static int vme_TestCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_register reg;
  vme_rc rc;
  int inst;
  if (argc != 2) return vme_TclArgError(interp,*argv,2);
  rc = vme_TclGetRegister(interp,*++argv,&reg,&inst); 
  if (vmeError(rc)) return vme_TclReturn(interp,rc); 
  return vme_TclRcReturn(interp,vmeTest(inst,reg));
}

static int vme_FileWriteCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_register reg;
  vme_rc rc;
  int inst, addr;
  if (argc != 4) return vme_TclArgError(interp,*argv,4);
  rc = vme_TclGetRegister(interp,*++argv,&reg,&inst); 
  if (Tcl_GetInt(interp,*++argv,&addr) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (vmeError(rc)) return vme_TclReturn(interp,rc); 
  return vme_TclRcReturn(interp, vmeFileWrite(inst,reg,addr,*++argv));
}

static int vme_FileReadCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_register reg;
  vme_rc rc;
  int inst, addr, len;
  if (argc != 5) return vme_TclArgError(interp,*argv,5);
  rc = vme_TclGetRegister(interp,*++argv,&reg,&inst); 
  if (Tcl_GetInt(interp,*++argv,&addr) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (Tcl_GetInt(interp,*++argv,&len) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (vmeError(rc)) return vme_TclReturn(interp,rc); 
  return vme_TclRcReturn(interp, vmeFileRead(inst,reg,addr,len,*++argv));
}

static int vme_pmCommandCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
  int inst,opcd,addr,chan,cnt,dir;
  int i,v,len;
  vme_vmetype *chip_data;
  char **listbase,**list;
  vme_rc rc;
  if (argc != 8) return vme_TclArgError(interp,*argv,8);
  if (Tcl_GetInt(interp,argv[1],&inst) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (Tcl_GetInt(interp,argv[2],&opcd) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (Tcl_GetInt(interp,argv[3],&addr) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (Tcl_GetInt(interp,argv[4],&chan) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (Tcl_GetInt(interp,argv[5],&cnt) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (Tcl_GetInt(interp,argv[6],&dir) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (Tcl_SplitList(interp,argv[7],&len,&listbase) != TCL_OK) {
    return vme_TclReturn(interp,VME_BAD_LIST);
  }
  list = listbase;
  chip_data = (vme_vmetype*) malloc(len * sizeof(vme_vmetype));
  /* be carful that the free() below is reached */
  /* also make sure to free() list */
  for (i = 0; i<len && (Tcl_GetInt(interp,*list++,&v) == TCL_OK) && v >= 0; i++) {
    chip_data[i] = (vme_vmetype)v;
  }
  free(listbase);
  if (i == len)
    rc = vmePmCommand(inst, opcd, addr,(vme_vmetype) chan,(vme_vmetype) cnt, dir, chip_data);
  else 
    rc = VME_BAD_VMETYPE;
  free(chip_data);
  return vme_TclRcReturn(interp,rc);
}

static int vme_BuffWriteCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_register reg;
  vme_rc rc = VME_OK;
  int inst;
  int i,len,v;
  vme_vmetype *buff;
  char **listbase,**list;
  if (argc != 3) return vme_TclArgError(interp,*argv,3);
  rc = vme_TclGetRegister(interp,*++argv,&reg,&inst); 
  if (vmeError(rc)) return vme_TclReturn(interp,rc); 
  if (Tcl_SplitList(interp,*++argv,&len,&listbase) != TCL_OK) {
    return vme_TclReturn(interp,VME_BAD_LIST);
  }
  list = listbase;
  buff = (vme_vmetype*) malloc(len * sizeof(vme_vmetype));
  /* be carful that the free() below is reached */
  /* also make sure to free() list */
  for (i = 0; i<len && (Tcl_GetInt(interp,*list++,&v) == TCL_OK) && v >= 0; i++) {
    buff[i] = (vme_vmetype)v;
  }
  free(listbase);
  if (i == len)
    rc = vmeBuffWrite(inst,reg,buff,len);
  else 
    rc = VME_BAD_VMETYPE;
  free(buff);
  return vme_TclRcReturn(interp,rc);
}

static int vme_BuffReadCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
  vme_register reg;
  vme_rc len,length,rc;
  int inst;
  vme_vmetype *buff,*buffbase;
  Tcl_DString dstring;
  if (argc != 3) return vme_TclArgError(interp,*argv,3);
  rc = vme_TclGetRegister(interp,*++argv,&reg,&inst); 
  if (vmeError(rc)) return vme_TclReturn(interp,rc); 
  if (Tcl_GetInt(interp,*++argv,&length) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  buffbase = buff = (vme_vmetype*) malloc(reg->size * sizeof(vme_vmetype));
  /* be carful that the free() below is reached */
  len = vmeBuffRead(inst,reg,buff,length);
  if (len > 0) {
    Tcl_DStringInit(&dstring);
    while (len--) Tcl_DStringAppendElement(&dstring,vme_Itoa(*buff++));
    Tcl_DStringResult(interp,&dstring);
    Tcl_DStringFree(&dstring);
  }
  free(buffbase);
  return vme_TclReturn(interp,0);
}

static int vme_SetServerCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
#ifdef FRONTEND
  return vme_TclReturn(interp, VME_NOT_DISTRIBUTED);
#else
  if (argc != 2) return vme_TclArgError(interp,*argv,2);
  if (**++argv == 0) return vme_TclReturn(interp, vme_UnsetServerHost());
  return vme_TclReturn(interp, vme_SetServerHost(*argv));
#endif
}

static int vme_UnsetServerCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
#ifdef FRONTEND
  return vme_TclReturn(interp, VME_NOT_DISTRIBUTED);
#else
  if (argc != 1) return vme_TclArgError(interp,*argv,1);
  return vme_TclReturn(interp, vme_UnsetServerHost());
#endif
}

static int vme_FrontDisplayCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
  if (argc != 2) return vme_TclArgError(interp,*argv,2);
  return vme_TclReturn(interp, vmeFrontDisplay(*++argv));
}

static int vme_AddCardCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
  vme_rc rc;
  int base;
  vme_card card;
  char* instname;
  int address;

  if (argc != 4) return vme_TclArgError(interp,*argv,4);

  card = vmeName2Card(argv[1]);
  if (vmeError(rc)) return vme_TclReturn(interp,rc);
  instname = argv[2];
  if (Tcl_GetInt(interp,argv[3],&base) != TCL_OK) return vme_TclReturn(interp,VME_BAD_ADDRESS);

  address = (int)vmeAddCard(card, instname, (vme_vmetype*)base);

  vme_TclBaseAdd(interp, card);

  return vme_TclRcReturn(interp, address);
}

static int vme_AddCardTypeCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
  if (argc != 2) return vme_TclArgError(interp,*argv,2);
  vmeAddCardType(argv[1]);
  return vme_TclReturn(interp,0);
}

static enum vme_regtype str2type(char* type)
{
  if (!strcmp(type,"bit"))
    {
      return VME_BIT;
    }
  else if (!strcmp(type,"bits"))
    {
      return VME_BITS;
    }
  else if (!strcmp(type,"word"))
    {
      return VME_WORD;
    }
  else if (!strcmp(type,"fifo"))
    {
      return VME_FIFO;
    }
  else if (!strcmp(type,"contig"))
    {
      return VME_CONTIG;
    }
  else
    {
      return VME_OTHER;
    }
}

static enum vme_access str2access(char* access)
{
  enum vme_access flags = 0;
  int len;

  len = strlen(access);

  while(len--)
    {
      if (access[len] == 'r' || access [len] == 'R')
	{
	  flags |= VME_R_ACCESS;
	}
      else if (access[len] == 'w' || access[len] == 'W')
	{
	  flags |= VME_W_ACCESS;
	}
      else if (access[len] == 'v' || access[len] == 'V')
	{
	  flags |= VME_V_ACCESS;
	}
      else if (access[len] == 'd' || access[len] == 'D')
	{
	  flags |= VME_D_ACCESS;
	}
      else if (access[len] == 's' || access[len] == 'S')
	{
	  flags |= VME_S_ACCESS;
	}
    }

  return flags;
}

static int vme_AddRegisterCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
  vme_rc rc;
  
  vme_register reg = 0;
  char* name;
  char* class;
  enum vme_regtype type;
  enum vme_access access;
  int offset;
  int mask;
  int vmask;
  int shift;
  vme_card card;

  if (argc != 10) return vme_TclArgError(interp,*argv,10);

  name = (char*) malloc(strlen(argv[1]) + 1);    /* lets be lazy and never reclaim this space */
  strcpy(name,argv[1]);
			
  class = (char*) malloc(strlen(argv[2]) + 1);
  strcpy(class,argv[2]);
			
  type = str2type(argv[3]);
  access = str2access(argv[4]);
  
  if (Tcl_GetInt(interp,argv[5],&offset) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (Tcl_GetInt(interp,argv[6],&mask) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (Tcl_GetInt(interp,argv[7],&vmask) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (Tcl_GetInt(interp,argv[8],&shift) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  card = vmeName2Card(argv[9]);
  if (vmeError(0)) return vme_TclReturn(interp,0);

  vmeAddRegister(&reg, name, class, type, access, offset, (vme_vmetype)mask, (vme_vmetype)vmask, shift, card);

  vme_TclRegAdd(interp, reg);

  return vme_TclReturn(interp, VME_OK);
}

/* used like so: vme_wait register delay(in 1/60 sec) [pole_value == 0 or 1] */
static int vme_WaitCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
  int pole = 1;
  int ticks;  /* one tick is 1/60 of a second */
  vme_register reg;
  vme_rc rc;
  int inst;
  if (argc != 3 && argc != 4 ) 
    return vme_TclError(interp,"usage: vme_Wait reg time [pole_value]",VME_WRONG_NUMBER_ARGS);
  rc = vme_TclGetRegister(interp,*++argv,&reg,&inst); 
  if (vmeError(rc)) return vme_TclReturn(interp,rc); 
  if (Tcl_GetInt(interp,*++argv,&ticks) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if ((argc == 4) && (Tcl_GetInt(interp,*++argv,&pole) != TCL_OK)) return vme_TclReturn(interp,VME_BAD_NUMBER);
  return vme_TclReturn(interp,vmeWait(inst,reg,pole,ticks));
}

static char* addv(char* s){
  static char r[256];
  sprintf(r,"%sv",s);
  return r;
}

static vme_rc GetState(Tcl_Interp* interp, int class, int inst)
{
  vme_register *registers,*reg;
  vme_vmetype *values,*val;
  int count;
  registers = reg = (vme_register*) malloc(VME_Reg.number*sizeof(vme_register));
  values = val = (vme_vmetype*) malloc(VME_Reg.number*sizeof(vme_vmetype));
  /* be carful to reach the free() func's below */
  count = vmeGetState(inst,class,registers,values);
  /* count might be an error # instead of a count */
  if (!vmeError(count)) {
    while(count--) {

      Tcl_SetVar2(interp,addv((*reg)->card->instname[inst]),(*reg)->name,vme_Itoa(*val++),TCL_GLOBAL_ONLY|TCL_LEAVE_ERR_MSG); 

      reg++;
    }
  }
  free(registers);
  free(values);
  /* count will be an error # or it will be zero */
  return count;
}

static int vme_GetStateCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_card card;
  vme_rc rc;
  int class,inst;
  if (argc != 2 && argc != 1) 
    {
      return vme_TclReturn(interp,vmeErrorReturn2("Too many arguments (use one or zero)",VME_WRONG_NUMBER_ARGS));
    }
  if (argc == 1) 
    {
      for (class = 0; VME_Cards[class] != 0; class++)
	{
	  for (inst = 0; inst < VME_Cards[class]->bases; inst++)
	    {
	      GetState(interp,class, inst);
	      if (vmeErrorState())
		{
		  return vme_TclReturn(interp,0);
		}
	    }
	}
    }
  else
    {
      card = vmeName2Card(*++argv);
      if (vmeErrorState()) 
	{
	  return vme_TclReturn(interp,0);
	}
      else
	{
	  inst = vmeName2Inst(*argv);
	  return vme_TclRcReturn(interp,GetState(interp, card->index, inst));
	}
    }
}

static int vme_ResetCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_rc rc;
  if (argc != 1) return vme_TclArgError(interp,*argv,1);
  if (vmeError(rc = vmeResetFront())) return vme_TclReturn(interp,rc);
  return vme_TclReturn(interp,VME_OK);
}

static int vme_VmeWriteCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  char errmess[64];
  int base,offset,value;
  sprintf(errmess,"usage: %s <base> [offset] <value>",*argv);
  if (argc != 3 && argc != 4) return vme_TclError(interp,errmess,VME_WRONG_NUMBER_ARGS);
  if (Tcl_GetInt(interp,*++argv,&base) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (argc == 4) {
    if (Tcl_GetInt(interp,*++argv,&offset) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  } else {
    offset = 0;
  }
  if (Tcl_GetInt(interp,*++argv,&value) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  return vme_TclReturn(interp, vmeVmeWrite((vme_vmetype*)base,offset,(vme_vmetype)value));
}

static int vme_VmeReadCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  char errmess[64];
  int base,offset;
  sprintf(errmess,"usage: %s <base> [offset]",*argv);
  if (argc != 2 && argc != 3) return vme_TclError(interp,errmess,VME_WRONG_NUMBER_ARGS);
  if (Tcl_GetInt(interp,*++argv,&base) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  if (argc == 3){
    if (Tcl_GetInt(interp,*++argv,&offset) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  } else {
    offset = 0;
  }
  return vme_TclRcReturn(interp, vmeVmeRead((vme_vmetype*)base,offset));
}

static int vme_DebugCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  int value;
  if (argc != 2) return vme_TclArgError(interp,*argv,2);
  if (Tcl_GetInt(interp,*++argv,&value) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
  return vme_TclReturn(interp,vmeDebug(value));
}


/* motor control functions..........*/

static int mxcCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  int value;
  static char return_result[100];
  if (argc != 2) return vme_TclArgError(interp,*argv,4);
/* if (Tcl_GetInt(interp,argv[1],&axis) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER);
 if (Tcl_GetInt(interp,argv[3],&cmd) != TCL_OK) return vme_TclReturn(interp,VME_BAD_NUMBER); */
  value = mExec(argv[1]);
  sprintf(return_result,"%d",value);
  interp->result = return_result;
  return TCL_OK;
}

static int mstatCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  int imotor, response, on_or_off;
  if (argc != 2) return vme_TclArgError(interp,*argv,4);
  if (Tcl_GetInt(interp,argv[1],&imotor) != TCL_OK) {
     interp->result = "wrong number of arguments to mstatus";
     return TCL_ERROR;
   }
   response  = mStat(imotor);
   on_or_off = response & 0x2;

  if (imotor == 1 && on_or_off == 0) Tcl_SetVar(interp,"stat1","OFF",0);
  if (imotor == 1 && on_or_off != 0) Tcl_SetVar(interp,"stat1","ON",0);
  if (imotor == 2 && on_or_off == 0) Tcl_SetVar(interp,"stat2","OFF",0);
  if (imotor == 2 && on_or_off != 0) Tcl_SetVar(interp,"stat2","ON",0);
  sprintf(interp->result, "%d", response);
  return TCL_OK;
}

/*static int mxgetCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  Tcl_SetVar(interp,"x",mXpos(),0);
   return TCL_OK;
}
*/

static int mdwrCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
   if(argc<2){ 
     printf("usage: mdwr command\n");
     return TCL_ERROR;
   }
   oPort(argv[1]);
   return TCL_OK;
}

static int mdrdCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
    char temp[256];
    char * iPort();
    strcpy(temp,iPort());
    /* printf("client:reply = %s\n",temp); */
    Tcl_SetVar(interp,"reply",temp,0);
   return TCL_OK;
}





static int vme_ResourceCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[])
{
  int value;
  if (argc != 3) return vme_TclArgError(interp,*argv,3);
  if (!(strcmp(argv[2],"teststand"))){
    if (strcmp(argv[1],"lock") == 0) return vme_TclReturn(interp,vmeLockResource(VME_RES_TESTSTAND,getpid()));
    if (strcmp(argv[1],"unlock") == 0) return vme_TclReturn(interp,vmeUnlockResource(VME_RES_TESTSTAND,getpid()));
    return vme_TclReturn(interp,vmeErrorReturn1("must use lock or unlock"));
  }
  return vme_TclReturn(interp,VME_RESOURCE_DNE);
}
 
 

/*
 * actual overall initialization procedure (called from TclAppInit)
 *
 */

int VME_Low_Mod_Init(Tcl_Interp* interp){
  vme_rc rc;
  Tcl_CreateCommand(interp, "vme_write",vme_WriteCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_read",vme_ReadCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_set",vme_SetCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_clear",vme_ClearCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_test",vme_TestCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_filewrite",vme_FileWriteCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_fileread",vme_FileReadCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_buffwrite",vme_BuffWriteCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_buffread",vme_BuffReadCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_setserver",vme_SetServerCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_unsetserver",vme_UnsetServerCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_reset",vme_ResetCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_frontdisplay",vme_FrontDisplayCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_addcard",vme_AddCardCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_addcardtype",vme_AddCardTypeCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_getstate",vme_GetStateCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_wait",vme_WaitCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_vmewrite",vme_VmeWriteCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_vmeread",vme_VmeReadCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_debug",vme_DebugCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_resource",vme_ResourceCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_addRegister",vme_AddRegisterCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "vme_pmCommand",vme_pmCommandCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
 
/* create motor control commands.........*/

 Tcl_CreateCommand(interp, "mxc",mxcCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  Tcl_CreateCommand(interp, "mstatus",mstatCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  /* Tcl_CreateCommand(interp, "mxget",mxgetCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
   */ 
   Tcl_CreateCommand(interp, "mdwr",mdwrCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
   Tcl_CreateCommand(interp, "mdrd",mdrdCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);

  vmeInitCards();

  if (!vmeErrorState())
    vmeInitRegs();

  if (!vmeErrorState())
    rc = vme_TclRegInit(interp);

  return vme_TclReturn(interp,rc);
}



