/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include <tk.h>
#include "regtrace.h"
#include "error.h"
#include "reglist.h"
#include "cardlist.h"
#include "cwish.h"
#include "tclutils.h"

/* infomod defines just one tcl command for getting information
 * about the system and cards and registers in it: vme_info
 *
 */

static int badusage(Tcl_Interp* interp, char* name, char* error){
  char mess[256];
  sprintf(mess,"funky usage: should be \"%s %s\"",name,error);
  return vme_TclError(interp,mess,VME_WRONG_NUMBER_ARGS);
}

static int streq(char* a, char* b){
  return !strcmp(a,b);
}

static int nullinfo(Tcl_Interp* interp, Tcl_DString* dstr, char* cmd, char* opt)
{
  int i,j;
  char* authors = "Eric Gottlieb, Tim Thomas, and Micheal Gold";
  
  char* information = 
"\n\
  VME based DAQ Test Stand Control Shell\n\
\n\
  type \"vme_info copyright\" for copyright information\n\
";

  char* copyright = 
"\n\
  Copyright 1994 by The University of New Mexico\n\
\n\
  Work supported by US DOE Contract #DE-FG-FG03-92ER40732\n\
\n\
  Design and Code by:\n\
    Eric Gottlieb  (egott@unm.edu)\n\
    Tim Thomas     (thomas@hev1.phys.unm.edu)\n\
    Micheal Gold   (gold@hepv1.phys.unm.edu)\n\
    edited by Armin Karcher (akarcher@lbl.gov)\n\
";

  if      (streq(opt,"cards"))
    {
      for (i = 0; VME_Cards[i] != NULL; i++)
	{
	  for (j = 0; j < VME_Cards[i]->bases; j++)
	    {
	      Tcl_DStringAppendElement(dstr,VME_Cards[i]->instname[j]);
	    }
	}
    }
  else if (streq(opt,"cardclasses"))
    {
      for (i = 0; VME_Cards[i] != NULL; i++)
	{
	  Tcl_DStringAppendElement(dstr,VME_Cards[i]->classname);
	}
    }
  else if (streq(opt,"author"))    Tcl_DStringAppendElement(dstr,authors);
  else if (streq(opt,"authors"))   Tcl_DStringAppendElement(dstr,authors);
  else if (streq(opt,"copyright")) Tcl_DStringAppendElement(dstr,copyright);
  else if (streq(opt,"info"))      Tcl_DStringAppendElement(dstr,information);
  else if (streq(opt,"commands")) {
    Tcl_DStringStartSublist(dstr);
    Tcl_DStringAppendElement(dstr,"write read set clear test");
    Tcl_DStringAppendElement(dstr,"filewrite fileread buffwrite buffread");
    Tcl_DStringAppendElement(dstr,"setserver unsetserver initialize reset addcard getstate");
    Tcl_DStringAppendElement(dstr,"wait vmewrite vmeread debug frontdisplay");
    Tcl_DStringEndSublist(dstr);
  } else {
    return badusage(interp,cmd,"commands");
  }
  return TCL_OK;
}

static vme_rc cardinfo(Tcl_Interp* interp, Tcl_DString* dstr, char* cmd, char* opt, vme_card card){
  int i;

  if      (streq(opt,"name"))  Tcl_DStringAppendElement(dstr,card->classname);
  else if (streq(opt,"type"))  Tcl_DStringAppendElement(dstr,card->classname);
  else if (streq(opt,"bases")) Tcl_DStringAppendElement(dstr,vme_Itoa((int)card->base[i]));
  else if (streq(opt,"base")) {
    Tcl_DStringStartSublist(dstr);
    for (i = 0; i < card->bases; i++) 
      Tcl_DStringAppendElement(dstr,vme_Htoa((int)card->base[i]));
    Tcl_DStringEndSublist(dstr);
  } else {
    return badusage(interp,cmd,"{name | type | bases | base} {card [card [...]]}");
  }
  return TCL_OK;
}

static int reginfo(Tcl_Interp* interp, Tcl_DString* dstr, char* cmd, char* opt, vme_register reg, int inst)
{
  if      (streq(opt,"name"))      Tcl_DStringAppendElement(dstr,reg->name);
  else if (streq(opt,"class"))     Tcl_DStringAppendElement(dstr,reg->class);
  else if (streq(opt,"card"))      Tcl_DStringAppendElement(dstr,reg->card->classname);
  else if (streq(opt,"cardinst"))  Tcl_DStringAppendElement(dstr,reg->card->instname[inst]);
  else if (streq(opt,"type"))      Tcl_DStringAppendElement(dstr,vmeRegTypeString(reg->type));
  else if (streq(opt,"offset"))    Tcl_DStringAppendElement(dstr,vme_Itoa(reg->offset));
  else if (streq(opt,"index"))     Tcl_DStringAppendElement(dstr,vme_Itoa(reg->index));
  else if (streq(opt,"mask"))      Tcl_DStringAppendElement(dstr,vme_Htoa(reg->mask));
  else if (streq(opt,"vmask"))     Tcl_DStringAppendElement(dstr,vme_Htoa(reg->vmask));
  else if (streq(opt,"shift"))     Tcl_DStringAppendElement(dstr,vme_Itoa(reg->shift));
  else if (streq(opt,"size"))      Tcl_DStringAppendElement(dstr,vme_Itoa(reg->size ? reg->size : 1));
  else if (streq(opt,"fullreg")){
    if (reg->fullreg) Tcl_DStringAppendElement(dstr,reg->fullreg->name);
    else              Tcl_DStringAppendElement(dstr,"none");
  }
  else if (streq(opt,"emptyreg")){
    if (reg->emptyreg) Tcl_DStringAppendElement(dstr,reg->emptyreg->name);
    else              Tcl_DStringAppendElement(dstr,"none");
  }
  else if (streq(opt,"access")) {
    Tcl_DStringStartSublist(dstr);
    if (reg->access & VME_W_ACCESS) Tcl_DStringAppendElement(dstr,"write");
    if (reg->access & VME_R_ACCESS) Tcl_DStringAppendElement(dstr,"read");
    if (reg->access & VME_V_ACCESS) Tcl_DStringAppendElement(dstr,"volatile");
    if (reg->access & VME_D_ACCESS) Tcl_DStringAppendElement(dstr,"decode");
    if (reg->access & VME_S_ACCESS) Tcl_DStringAppendElement(dstr,"special");
    Tcl_DStringEndSublist(dstr);
  }
  else if (streq(opt,"enabled")){
    Tcl_DStringStartSublist(dstr);
    if (reg->access & VME_W_ACCESS){ 
      if (reg->type == VME_BIT){
	Tcl_DStringAppendElement(dstr,"set");
	if (!(reg->access & VME_V_ACCESS)) 
	  Tcl_DStringAppendElement(dstr,"clear");
      } else {
	if (reg->type == VME_FIFO || reg->type == VME_CONTIG) {
	  Tcl_DStringAppendElement(dstr,"buffwrite");
	  Tcl_DStringAppendElement(dstr,"filewrite");
	} else
	  Tcl_DStringAppendElement(dstr,"write");
      }
    }
    if (reg->access & VME_R_ACCESS){
      if (reg->type == VME_BIT){
	Tcl_DStringAppendElement(dstr,"test");
      } else {
	if (reg->type == VME_FIFO || reg->type == VME_CONTIG) {
	  Tcl_DStringAppendElement(dstr,"buffread");
	  Tcl_DStringAppendElement(dstr,"fileread");
	} else
	  Tcl_DStringAppendElement(dstr,"read");
      }
    }
    Tcl_DStringEndSublist(dstr);
  }
  else if (streq(opt,"address")) {
    Tcl_DStringAppendElement(dstr,vme_Htoa((int)&reg->card->base[0][reg->offset]));
  }
  else{
    return badusage(interp,cmd,"{name | type | offset | address | ... } {reg [reg [...]]}");
  }
  return TCL_OK;
}


static int vme_InfoCmd(ClientData clientData, Tcl_Interp* interp, int argc, char* argv[]){
  vme_register reg;
  vme_card card;
  char *command,*option;
  int tcl_return_code = TCL_OK;
  vme_rc rc = VME_OK;
  int inst;
  Tcl_DString dstring; /* special tcl string: useful for returning results */

  command = *argv++; /* tcl name of info command */
  option = *argv++; /* option to info command (name, address, etc...) */
  if (option == NULL) return badusage(interp,command,"option [argument [...]]");
  Tcl_DStringInit(&dstring); /* make sure that Tcl_DStringFree() is reached below */
  do { /* *argv contains the parameter (NULL, a register, or a card) */
    if (*argv == NULL) { /* no parameter */
      tcl_return_code = nullinfo(interp,&dstring,command,option);
    } else {
      card = vmeName2Card(*argv);
      if (!vmeGetError()) { /* parameter is a card name */
	tcl_return_code = cardinfo(interp,&dstring,command,option,card);
      } else {
	rc = vme_TclGetRegister(interp,*argv,&reg,&inst);
	if (vmeError(rc))
	  tcl_return_code = vme_TclReturn(interp,rc);
	else
	  tcl_return_code = reginfo(interp,&dstring,command,option,reg,inst);
      }
    }
  } while ((*argv != NULL) && (*++argv != NULL) && (tcl_return_code == TCL_OK));
  if (tcl_return_code == TCL_OK)
    Tcl_DStringResult(interp,&dstring);
  Tcl_DStringFree(&dstring);
  return tcl_return_code;
}

int VME_Info_Mod_Init(Tcl_Interp* interp)
{
  Tcl_CreateCommand(interp, "vme_info",vme_InfoCmd,(ClientData) NULL,(Tcl_CmdDeleteProc *) NULL);
  return TCL_OK;
}

