/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include <stdio.h>
#include <sys/types.h>

#ifdef vxworks
#include <types/vxTypesOld.h>
#endif

#include "error.h"
#include "cardlist.h"
#include "reglist.h"
#include "user.h"
#include "vmemap.h"

#ifdef FAKECARD
#include "fake.h" /* for vmeFakeBase() */
#endif

#define VME_MAX_CARDS 32

vme_card VME_Cards[VME_MAX_CARDS];

static int cardCounter = 0;  /* used only by vmeAddCardClass and vmeDeleteCards */
                             /* which are the only routines that can't count on */
                             /* the null termination.                           */

void vmeAddCardClass(vme_card* c, char* name)
{
  *c = (vme_card) malloc(sizeof(struct vme_card));
  VME_Cards[cardCounter] = *c;

  (*c)->index = cardCounter;
  (*c)->bases = 0;
  (*c)->classname = (char*) malloc(strlen(name) + 1);
  strcpy((*c)->classname,name);

  VME_Cards[++cardCounter] = NULL;

  printf("added new card class:\n");
  printf("index = %d, bases = %d, classname = %s, cardCounter = %d\n",(*c)->index, (*c)->bases, (*c)->classname, cardCounter);
  
}

static deleteCard(vme_card c)
{
  int i;

  free(c->classname);       /* get rid of the space allocated for classname */
  for (i = 0; i < c->bases; i++)
    {
      free(c->instname[i]); /* get rid of the space allocated for instname */
    }
  free(c);                  /* get rid of the card itself */
}

/* 
   if a clean way to shut down the server ever gets provided, it should
   call vmeDeleteCards() and vmeDeleteRegs().
*/

void vmeDeleteCards()
{
  int i;

  for (i = 0; i < cardCounter; i++)
    {
      deleteCard(VME_Cards[i]);
    }

  cardCounter = 0;
}

void vmeInitCards()
{
/* get rid of old cards (incase we have already been here before */

  vmeDeleteCards();

/* now add the hard coded cards */

  vmeAddUserCards();
}

int vmeBase2Index(vme_vmetype* base)
{
  int class;
  int index;
  vme_card card;

  for (class = 0; VME_Cards[class] != NULL; class++)
    {
      card = VME_Cards[class];
      for (index = 0; index < card->bases; index++)
	{
	  if (card->base[index] == base)
	    {
	      return index;
	    }
	}
    }

  vmeErrorReturn(VME_CARD_DNE);
  return 0;
}

vme_card vmeName2Card(char* name)
{
  int class;
  int inst;

  for (class = 0; VME_Cards[class] != NULL; class++)
    {
      /* test against classname */
      if (strcmp(VME_Cards[class]->classname,name) == 0)
	{
	  return VME_Cards[class];
	}
      /* test against instance names */
      for (inst = 0; inst < VME_Cards[class]->bases; inst++)
	{
	  if (strcmp(VME_Cards[class]->instname[inst],name) == 0)
	    {
	      return VME_Cards[class];
	    }
	}
    }
  
  vmeErrorReturn(VME_CARD_DNE);
  return NULL;
}

int vmeName2Inst(char* name)
{
  int class,inst;

  for (class = 0; VME_Cards[class] != NULL; class++)
    {
      /* test against instance names */
      for (inst = 0; inst < VME_Cards[class]->bases; inst++)
	{
	  if (strcmp(VME_Cards[class]->instname[inst],name) == 0)
	    {
	      return inst;
	    }
	}
    }
  
  vmeErrorReturn(VME_CARD_DNE);
  return 0;
}


static vme_vmetype* vmemap(vme_vmetype* address)
{
if ( address < (vme_vmetype*)0x70000000 )
  {
    address = (vme_vmetype*)((unsigned int)address | MAPSHIFT);
   }
  return address;
}

vme_vmetype* vmeAddCardBase(vme_card card, char* instname, vme_vmetype* base)
{
  static char dummybase[4096];
  static int dummyindex = 0;

  printf("vmeAddCardBase:\n");
  printf("vmeAddCardBase: card = %s, instname = %s, base = %d\n",card->classname, instname, base);

  if ((unsigned int)base == 1) /* make up a dummybase */
    {
      base = (vme_vmetype*) &dummybase[(dummyindex++)*512];
#ifdef FAKECARD
      if (card->index == 0) 
	{ 
	  /* assume the first one is the fake one and
	     use the predefined fake base instead */
	  base = vmeGetFakeBase();
	  dummyindex--;
	}
#endif
    }
  else 
    {
      base = vmemap(base);
    }

  if ((unsigned int)base%4) 
    {
      vmeErrorReturn(VME_UNALIGNED_ADDRESS);
      return 0;
    }
  
  if (1 || debugging) 
    {
      printf("vmeAddCardBase: instname = %s, dummybase = 0x%8.8x and base = 0x%8.8x\n",instname, (unsigned int)dummybase,(unsigned int)base);
    }
  
  vmeBase2Index(base);
  if (vmeGetError() != VME_CARD_DNE) 
    {
      vmeErrorReturn(VME_CARD_EXISTS);
      return 0;
    }
  
  card->base[card->bases] = base;
  card->instname[card->bases] = (char*) malloc(strlen(instname) + 1);
  strcpy(card->instname[card->bases],instname);
  card->bases++;
  return base;
}


