/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

/* $Id: error.c,v 1.1.1.1 1995/05/05 16:16:17 egott Exp $ */

#include <stdio.h>
#include <unistd.h> /* for sleep */
#ifdef vxworks
#include <semLib.h>
#endif

#include "semaphore.h"
#include "error.h"

/*
 * error.c
 *
 * Refer to the documentation for more information about the global
 * symbols (external functions for the most part) defined here.
 * error.h also has some informative comments.
 *
 */

/*
 * the global variable "debugging" is currently treated as a boolean.  It can
 * be set or cleared w/ vmeDebug() from the front or back end.  If it is true
 * some extra information will be printed out on the front end console.
 */

int debugging = 0; /* debugging is on if not zero */

#ifdef FRONTEND
vme_rc vmeDebug(int level){
  debugging = level;
  return VME_OK;
}
#endif

#define MAXERRORS 128

static int errors[MAXERRORS];
static int tasks[MAXERRORS];
static int toperror = 0;

#ifdef vxworks

static int getpid () {
  return taskIdSelf();
}

#endif

static SEM_ID semaphore = 0;

static enum vme_error first_error(int pid){
  int i = 0;
  vmeSemLock(&semaphore);
  while (i < toperror && tasks[i] != pid)  i++;
  vmeSemUnlock(&semaphore);
  if (i >= toperror) 
    return 0;
  else
    return errors[i];  
}

enum vme_bool vmeErrorState(){
  if (toperror == 0) {
    return VME_OK;
    }
  else
    return (first_error(getpid()) != 0);
}

void add_error(enum vme_error e){
  vmeSemLock(&semaphore);
  if (debugging) printf( "added error %d \n",e);
  if (toperror < MAXERRORS - 1) {
    errors[toperror] = e;
    tasks[toperror] = getpid();
    toperror++;
  } 
  else if (toperror < MAXERRORS) {
    errors[toperror] = VME_TOO_MANY_ERRORS;
    tasks[toperror] = getpid();
  }
  vmeSemUnlock(&semaphore);
}

void delete_error(enum vme_error e,int pid) {
  int i = 0;
  int j;

  vmeSemLock(&semaphore);
  while(i < toperror && !(tasks[i] == pid && (errors[i] == e || e == 0))) i++;
  if (i < toperror) {
    for (j = i; j < toperror - 1; j++) {
      errors[j] = errors[j+1];
      tasks[j] = tasks[j+1];
    }
    toperror--;
  }    
  vmeSemUnlock(&semaphore);
} 

/* vmeError is used to determine if an error has occured.
 */

enum vme_bool vmeError(vme_rc e){
  return vmeErrorState();
}

/* static error number are ones that are defined in "enum vme_error" in error.h.
 * dynamic error numbers are created by calls to vmeErrorReturn1() or vmeErrorReturn2().
 *
 * static error numbers are positive and dynamic ones are negitive.
 */

/* the array in which dynamic error strings are kept */
static char* errstrings[512];

/* this points to the first free/empty error string in errstrings */
static int errindex = 0;

/* this is the first legal dynamic error number.  subsequent ones are more negative. */
const static int err_start = -100;

/* err2index takes a dynamic error number and returns the associated index into errstrings[] 
 * it does not test for out of range values.
 */
static int err2index(int err){
  return(err_start - err);
}

/* 
 * index2err takes an index into errstrings[] and returns the associated dynamic error number 
 * it does not test for out of range values.
 */
static int index2err(int idx){
  return(err_start - idx);
}

/* vmeErrorReturn() registers the fact than an error occured and returns the error code passed to it.
 *
 */
enum vme_error vmeErrorReturn(enum vme_error e){
  add_error(e);
  return e;
}

/* vmeErrorReturn1() returns a unique dynamicaly assigned error number which is associated (via errstrings[] and err2index)
 * with the message string passed to it.  If the string has already been passed to it (the contents of the strings
 * are compared, not the pointers), the error number dynamicly assigned earlier will be use and no new space will be allocated.
 * The passed string does not have to be static as it is copied in to localy malloced memory.
 */
enum vme_error vmeErrorReturn1(char* s){
  int len,i;
  enum vme_error err = 0;

  vmeSemLock(&semaphore);
  /* check to see if an equivlent string has already had space allocated for it
   * if so, simply return it's assocaited dynamic error number
   */
  for(i=0;i<errindex;i++) 
    if (!strcmp(s,errstrings[i])) 
      err = index2err(i);
  /* otherwise malloc space for a new error message and give it a new unique dynamic error number. */
  if (!err) {
    len = strlen(s);
    errstrings[errindex] = (char*) malloc((len+1)*sizeof(char));
    strcpy(errstrings[errindex],s);
    err = index2err(errindex++);
  }
  vmeSemUnlock(&semaphore);
  add_error(err);
  return err;
}

/* vmeErrorReturn2() works much like vmeErrorReturn1() above (it makes use of it), excpet that it combines the passed
 * message string with the "standard" string associated with the additional passed static error number.
 */
enum vme_error vmeErrorReturn2(char* s,enum vme_error err){
  char buffer[256];
  sprintf(buffer,"%s : %s",s,vmeGetErrorString(err));
  return vmeErrorReturn1(buffer);
}

/* it is neccisary to prototype this here to deal with a very mild case of recursion */
static char * num2string(enum vme_error err);

/* vmeErrorString, given an error number (static or dynamic), returns the associate
 * error message string
 */

char* vmeGetErrorString(enum vme_error err){
  return num2string(err);
}

static enum vme_error get_error(int pid){
  enum vme_error err;

  err = first_error(pid);
  delete_error(err,pid);
  return err;
}

enum vme_error vmeGetError(){
  return get_error(getpid());
}

/* for backwards compatibility */
char *vmeErrorString(enum vme_error err){
  enum vme_error temperr,lasterr;
  lasterr = vmeGetError();
  while (temperr = vmeGetError()) lasterr = temperr;
  return vmeGetErrorString(lasterr);
}

#ifdef FRONTEND 
char* vmeFrontError(int pid){
  enum vme_error err;

  err = get_error(pid);
  if (err)
    return vmeGetErrorString(err);
  else 
    return "";
}
#endif

/* num2string is used by vmeErrorString to retrieve the error message string associated
 * with an error number.  It's handling of static error numbers is not exactly elegant,
 * but it is simple and it allows for relitivly arbitrary static error numbers.
 */
static char *num2string(enum vme_error err){
  switch (err) {
  case VME_ILLEGAL_REGISTER : return("illegal/unknown register");
  case VME_ILLEGAL_QUERY : return("illegal query");
  case VME_ILLEGAL_FIFO : return("illegal fifo");
  case VME_ILLEGAL_PARAMETER : return("illegal parameter");
  case VME_NO_WRITE : return("not write enabled");
  case VME_NO_READ : return("not read enabled");
  case VME_NO_BITSWRITE : return("not bits_write enabled");
  case VME_NO_BITSREAD : return("not bits_read enabled");
  case VME_NO_SET : return("not bit set enabled");
  case VME_NO_CLEAR : return("not bit clear enabled");
  case VME_NO_TEST : return("not bit test enabled");
  case VME_NO_BUFFWRITE : return("not buff write enabled");
  case VME_NO_BUFFREAD : return("not buff read enabled");
  case VME_ILLEGAL_CARD : return("unknown card");
/*   case VME_ILLEGAL_CARD_ORDER : return("illegal card initialization order"); */
  case VME_TOO_MANY_CARDS : return("ran out of space in card list array");
  case VME_TOO_MANY_REGISTERS : return("ran out of space in register list array");
  case VME_WRONG_METHOD_FOR_OBJECT : return("wrong method for object was used");
  case VME_MISSING_METHOD : return("method does not exist");
  case VME_COULDNT_GIVE_SEM : return("could not give semaphore");
  case VME_COULDNT_TAKE_SEM : return("could not take semaphore"); 
  case VME_COULDNT_MAKE_SEM : return("could not make semaphore");
  case VME_RESOURCE_NOT_AVAILABLE : return("requested resource was not available");
  case VME_RESOURCE_DNE : return("requested resource does not exist");
  case VME_RESOURCE_WRONG_ID : return("requested resource already locked");
  case VME_SEMAPHORE_DNE : return("unlock attempted w/o semaphore");
  case VME_ILLEGAL_SEMGIVE : return("semgive failed");
  case VME_RESOURCE_NOT_LOCKED : return("requested resource already unlocked");
  case VME_CARD_DNE : return("card does not exist");
  case VME_WRONG_NUMBER_ARGS : return("wrong number of arguments");
  case VME_NOT_IMPLEMENTED_YET : return("not yet implemented");
  case VME_NO_SERVERHOST : return("host for server is unspecified");
  case VME_RPC_FAILED : return("failed to successfully issue rpc");
  case VME_NO_CLIENT : return("server does not exist");
  case VME_TIMEOUT : return("command timed out");
  case VME_BN_IN_PARALLEL_MODE : return("can not write BN in parallel mode");
  case VME_EMU_CLK_NOTOGGLE : return("can not toggle emulator clock");
  case VME_FRONTEND_ONLY : return("front end only routine called from back end");
  case VME_BAD_ENABLEDTYPE : return("illegal enabled function type");
  case VME_CANT_OPEN_FILE : return("can not open file");
  case VME_CANT_READ_FILE : return("can not read from file");
  case VME_CANT_WRITE_FILE : return("can not write to file");
  case VME_TIMED_OUT : return("function timed out");
  case VME_ILLEGAL_HDIADDR : return("illegal hdi address");
  case VME_ILLEGAL_NUMCHIPS : return("illegal number of chips");
  case VME_BAD_LIST : return("bad list");
  case VME_BAD_VMETYPE : return("bad number (vme_vmetype)");
  case VME_CANT_MALLOC : return("can not malloc (get memory)");
  case VME_CARD_EXISTS : return("card base address already being used");
  case VME_BAD_NUMBER : return("bad number (int)");
  case VME_BAD_ADDRESS : return("bad number (address)");
  case VME_UNALIGNED_ADDRESS : return("unaligned address");
  case VME_DEBUGGING : return("this is a forced debug message");
  case VME_INTERNAL : return("an unknown internal error has occurred");
  case VME_ERROR : return("an unexplained error has occurred");
  case VME_NOT_DISTRIBUTED : return("not a distributed system");
  case VME_NOT_IMPLEMENTED : return("attempted execution of unimplemented routine");
  case VME_FIFO_FULL : return("fifo overflow");
  case VME_FIFO_EMPTY : return("fifo underflow");
  case VME_BAD_CODE : return("bad bit pattern in decode");
  case VME_WRONG_MODE : return("illegal mode");
  case VME_WRONG_STATE : return("illegal state");
  case VME_SCRAMBLED_FIFO : return("fifo is scrambled");
  case VME_BAD_SRECORD : return("unreadable s-record");
  case VME_TOO_MANY_ERRORS : return("too many errors");
  case VME_ARRAY_OVERREAD : return("attempted read past array register boundry");
  case VME_ARRAY_OVERWRITE : return("attempted write past array register boundry");
  case VME_BUS_ERROR : return("Bus error.");
  case VME_BAD_ARGUMENT : return("unrecognized or missing argument");
  default :
    /* the error number is not recognized as a static error;
     * is it a legal dynamic error number?
     * if so get the string from errstrings[] 
     * otherwise the error is compleatly unknown
     */
    if (err2index(err) >= 0 && err2index(err) < errindex) 
      return(errstrings[err2index(err)]); 
    else
    return("unknown error (unknown error number)");
  }
}

/* sleep is used by vmeFatalError below 
 * vxworks doesn't define a sleep command, so I do here
 */
#ifdef vxworks
static int sleep(int seconds){
  return taskDelay(seconds*sysClkRateGet());
}
#endif

/* Some errors are sufficiently serious that the normal methods for handling them can not be trusted.
 * vmeFatalError reports the passed string and the error message associated with the second argument
 * and then stops the current task by exiting (after delaying for a few seconds).
 */

void vmeFatalError(char* who,vme_rc what){
  printf("\n");
  printf(">>---------->> fatal error in %s: %s. <<----------<<\n",who,vmeGetErrorString(what));
  printf("\n");
  printf("delaying 5 seconds....\n");
  sleep(5);
  printf("exiting.\n");
  exit(-1);
}

