/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

/* $Id: error.h,v 1.1.1.1 1995/05/05 16:16:17 egott Exp $ */

#ifndef error_h
#define error_h

/* error.h:
 *
 * definitions of return codes (vme_rc), error numbers (vme_error), 
 * booleans (vme_bool), and error handling procedures (see description 
 * of vme_rc just below).
 *
 */

/* vme_rc:
 *
 * vme_rc == our generic return code == integer (at least for now it does :).
 * While it is true that vme_rc is interchangable with int, using vme_rc seems
 * like a good idea since it helps to make the code self documenting.
 *
 * vme_rc can be interpreted as either being an integer or an error code.  vmeError()
 * can be used to tell the difference.  Currently error codes are always less than
 * -1, but this is not garanteed to always be true.  Ie: don't count on it.
 *
 * Most global routines don't need to return anything more complicated than a positive
 * integer...therefor vme_rc is used genericaly as a return value through out the vme
 * test stand code.
 *
 * a vme_rc can be tested with vmeError() to see if it is indeed an error code, and can
 * be passed to either vmeErrorString() or vmeErrorPrint() to return an appropriate
 * error string or print out the same to stderr respectivly.
 *
 */

typedef int vme_rc;

/* vme_error: 
 *
 * these error numbers are, for the most part, meant to corispond to relitivly
 * generic errors which could occure in more than one routine.  vmeErrorReturn1() 
 * can be used to report errors which are more unique. 
 *
 * when adding a new error number, remember to also modify vmeErrorString in
 * error.c if you want to define an assosiated error message.
 *
 */

enum vme_error {
  VME_FRONTEND_ERROR = 100,
  VME_ILLEGAL_REGISTER = 101,
  VME_ILLEGAL_QUERY = 102,
  VME_ILLEGAL_FIFO = 103,
  VME_ILLEGAL_PARAMETER = 104,
  VME_NO_WRITE = 105,
  VME_NO_READ = 106,
  VME_NO_SET = 107,
  VME_NO_CLEAR = 108,
  VME_NO_TEST = 109,
  
  VME_NO_CLIENT = 110,
  VME_NO_SERVERHOST =  111,
  VME_RPC_FAILED = 112,
  
  VME_ILLEGAL_CARD = 113,
/*   VME_ILLEGAL_CARD_ORDER = 114,  not used anymore */
  VME_TOO_MANY_CARDS = 115,
  VME_TOO_MANY_REGISTERS = 116,

  VME_WRONG_METHOD_FOR_OBJECT = 117,
  VME_MISSING_METHOD = 118,

  VME_COULDNT_GIVE_SEM = 119,
  VME_COULDNT_TAKE_SEM = 120,
  VME_COULDNT_MAKE_SEM = 121,
  VME_RESOURCE_NOT_AVAILABLE = 122,
  VME_RESOURCE_DNE = 123,
  VME_RESOURCE_WRONG_ID = 124,
  VME_SEMAPHORE_DNE = 125,
  VME_ILLEGAL_SEMGIVE = 126,
  VME_RESOURCE_NOT_LOCKED = 127,

  VME_CARD_DNE = 128,

  VME_WRONG_NUMBER_ARGS = 129,
  VME_NOT_IMPLEMENTED_YET = 130,

  VME_TIMEOUT = 131,
  VME_BN_IN_PARALLEL_MODE = 132,

  VME_NO_BITSWRITE = 133,
  VME_NO_BITSREAD = 134,

  VME_EMU_CLK_NOTOGGLE = 135,

  VME_FRONTEND_ONLY = 136,

  VME_BAD_ENABLEDTYPE = 137,

  VME_NO_BUFFWRITE = 138,
  VME_NO_BUFFREAD = 139,

  VME_CANT_OPEN_FILE = 140,
  VME_CANT_READ_FILE = 141,
  VME_CANT_WRITE_FILE = 142,

  VME_TIMED_OUT = 143,
  VME_ILLEGAL_HDIADDR = 144,
  VME_ILLEGAL_NUMCHIPS = 145,

  VME_BAD_LIST = 146,
  VME_BAD_VMETYPE = 147,
  VME_CANT_MALLOC = 148,

  VME_CARD_EXISTS = 149,
  VME_BAD_NUMBER = 150,
  VME_BAD_ADDRESS = 151,
  VME_UNALIGNED_ADDRESS = 152,

  VME_DEBUGGING = 153,
  VME_INTERNAL = 154,

  VME_NOT_DISTRIBUTED = 155,
  VME_NOT_IMPLEMENTED = 156,

  VME_FIFO_FULL = 157,
  VME_FIFO_EMPTY = 158,
  VME_BAD_CODE = 159,
  VME_WRONG_MODE = 160,
  VME_WRONG_STATE = 161,
  VME_SCRAMBLED_FIFO = 162,

  VME_BAD_SRECORD = 163,
  VME_TOO_MANY_ERRORS = 164,
  VME_ARRAY_OVERREAD = 165,
  VME_ARRAY_OVERWRITE = 166,
  VME_BUS_ERROR = 167,
  VME_BAD_ARGUMENT = 168,

  VME_ERROR = 500,
  VME_OK = 0
};

/* 
 * sometimes it is convienient to be explicit on ones use of booleans....
 * and sometimes it isn't.  enum vme_bool is provided for when it is.
 *
 */

enum vme_bool { VME_FALSE = 0, VME_TRUE = 1 };

/* these functions are describe in the documentation and in error.c */

enum vme_bool vmeError(vme_rc e);
enum vme_bool vmeErrorState();

char* vmeGetErrorString(enum vme_error err);
enum vme_error vmeGetError();

/* for backwards compatability */
char *vmeErrorString(enum vme_error err);

void vmeFatalError(char* who,vme_rc what);
enum vme_error vmeErrorReturn(enum vme_error err);
enum vme_error vmeErrorReturn1(char* s);
enum vme_error vmeErrorReturn2(char* s,enum vme_error err);

/* When this variable is not zero some extra debugging information is
 * printed out on the front end.  In theory the amount of information
 * displayed could depend on the value of debugging, but currently
 * it is just used as a boolean.  It is not clear weather this will
 * always be part of the system or not.  For not it is just a hack.
 */

extern int debugging; 

/* this routine can be used to set or clear the global variable "debugging" */
vme_rc vmeDebug(int level);

/* for internal use */
char* vmeFrontError(int pid);

#endif

