/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (Karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include <stdio.h>
#include <sys/types.h>

#ifdef vxworks
#include <types/vxTypesOld.h>
#include <wdLib.h>
#endif

#include "reglist.h"
#include "front.h"
#include "error.h"
#include "low.h"
#include "semaphore.h"
#include "executor.h"
#include "rpc.h"
#include "vmemap.h"

vme_rc vmeResetFront()
{
  vmeInitCards();					      
  vmeInitRegs();                                       
  if (vmeErrorState()) vmeFatalError("vmeResetFront",0);
  return VME_OK;
}

vme_rc vmeWrite(int c,vme_register reg,vme_vmetype v){
  if (c < 0 || c >= reg->card->bases) return vmeErrorReturn(VME_CARD_DNE);
  return reg->write(reg->card->base[c],reg,v);
}

vme_rc vmeRead(int c,vme_register reg){
  if (c < 0 || c >= reg->card->bases) return vmeErrorReturn(VME_CARD_DNE);
  return reg->read(reg->card->base[c],reg);
}


vme_rc vmeSet(int c,vme_register reg){
  if (c < 0 || c >= reg->card->bases) return vmeErrorReturn(VME_CARD_DNE);
  return reg->set(reg->card->base[c],reg);
}

vme_rc vmeClear(int c,vme_register reg){
  if (c < 0 || c >= reg->card->bases) return vmeErrorReturn(VME_CARD_DNE);
  return reg->clear(reg->card->base[c],reg);
}

vme_rc vmeTest(int c,vme_register reg){
  if (c < 0 || c >= reg->card->bases) return vmeErrorReturn(VME_CARD_DNE);
  return reg->test(reg->card->base[c],reg);
}

vme_rc vmeBuffWrite(int c, vme_register reg, vme_vmetype* data, int len){
  vme_rc offset = 0;
  if (debugging) printf("front: vmeBuffWrite: reg = %s len = %d\n",reg->name,len);
  if (c < 0 || c >= reg->card->bases) return vmeErrorReturn(VME_CARD_DNE);
  return reg->buffwrite(reg->card->base[c],reg,data,len,offset);
}

vme_rc vmeBuffRead(int c, vme_register reg, vme_vmetype* data, int len){
  vme_rc offset = 0;
  if (debugging) printf("front: vmeBuffRead: reg = %s len = %d\n",reg->name,len);
  if (c < 0 || c >= reg->card->bases) return vmeErrorReturn(VME_CARD_DNE);
  return reg->buffread(reg->card->base[c],reg,data,len,offset);
}

static vme_vmetype* vmemap(vme_vmetype* address) 
{
  if ( address < (vme_vmetype*)0x70000000 )
  {
    address = (vme_vmetype*)((unsigned int)address | MAPSHIFT);
   }
  return address;
}


vme_rc vmeVmeBuffRead(vme_vmetype* base, int offset, vme_vmetype* data, int length){
  int i;
  base = vmemap(base);
  if (debugging)  printf("Reading %d words starting at 0x%8.8x \n",length,&base[offset]);
  for (i=0; i<length; i++) {
      data[i] = base[i+offset];
  }
  return length;
}


/*--------------------------------------------------------------------------*/
/* same as vmeVmeBuffRead, but skip every other word...ugly but should work */
/*--------------------------------------------------------------------------*/
vme_rc vmeVmeBuffRead2(vme_vmetype* base, int offset, vme_vmetype* data, int length){
  int i;
  int j;
  base = vmemap(base);
  j = 0;
  if (debugging)  printf("Reading %d words skipping every other one starting at 0x%8.8x \n",length,&base[offset]);
  for (i=0; i<length; i++) {
      data[i] = base[j+offset];
      j = j+2;
  }
  return length;
}

vme_rc vmeFrontDisplay(char* mess){
  printf("::> %s\n",mess); 
  return VME_OK;
}

vme_rc vmeGetState(int c, int classindex, vme_register* registers, vme_vmetype* values){
  int i,count = 0;
  vme_rc rc;
  vme_register* reg = VME_Reg.list;
  for (i=0;i<VME_Reg.number;i++)
    if ((reg[i]->card->index == classindex) 
	&& (reg[i]->access & VME_R_ACCESS) 
	&& (reg[i]->type != VME_FIFO) 
	&& (reg[i]->type != VME_CONTIG)){
      registers[count] = reg[i];
      if (vmeError(rc = vmeRead(c,reg[i]))) return rc;
      values[count] = rc;
      count++;
    }
  return count;
}

vme_vmetype* vmeAddCard(vme_card card, char* instname, vme_vmetype* base)
{
  return vmeAddCardBase(card,instname,base);
}

void vmeAddCardType(char* name)
{
  vme_card dummy;
  vmeAddCardClass(&dummy, name);
}

#ifdef vxworks
/* 
 * I think a tick is 1/60 of a second and vxworks seems to agree.
 * this is here in case it ever changes its mind
 *
 */
static int ticks2ticks(double ticks){
  return (int)(sysClkRateGet()*(ticks/60.0));  /* sysClkRateGet returns ticks/sec */
}

static int timeout(int flagptr){
  *(int*)flagptr = 1;
  return NULL;
}
#endif

vme_rc vmeWait(int card, vme_register bit_reg, vme_vmetype pole, int ticks){
#ifdef vxworks
  int timed_out = 0;
  vme_rc rc;
  WDOG_ID watchdog;
  if (debugging) printf("vmeWait %d ticks (%d vxticks) on %s = %d",ticks,ticks2ticks(ticks),bit_reg->name,pole);
  watchdog = wdCreate();
  wdStart(watchdog,ticks2ticks(ticks),timeout,(int)&timed_out);
  while (((rc = vmeTest(card,bit_reg)) == pole) & !timed_out);
  if (debugging) printf("done: timed_out = %d\n",timed_out);
  if (!timed_out) wdCancel(watchdog);
  wdDelete(watchdog);
  if (vmeError(rc)) return rc;
  if (timed_out) return vmeErrorReturn(VME_TIMED_OUT);
#endif
  return VME_OK;
}


vme_rc vmeVmeWrite(vme_vmetype* base,int offset,vme_vmetype value){
  base = vmemap(base);
  if (debugging) printf("0x%8.8x[%2.2d] = 0x%8.8x <- 0x%4.4x\n",base,offset,&base[offset],value);
#ifdef vxworks
  base[offset] = value;
#endif
  return VME_OK;
}

vme_rc vmeVmeRead(vme_vmetype* base,int offset){
  vme_vmetype value;
  base = vmemap(base);
#ifdef vxworks
  if (debugging) {
    value = base[offset];
    printf("0x%8.8x[%2.2d] = 0x%8.8x -> 0x%4.4x\n",base,offset,&base[offset],value);
    return value;
  }
  else
    return base[offset];
#else
  if (debugging) printf("0x%8.8x[%2.2d] = 0x%8.8x -> 0x%4.4x\n",base,offset,&base[offset],23/* base[offset] */);
  return 23 /* base[offset] */;
#endif
}

/* in future save task id here and use that for flag */

vme_rc vmeExecControl(struct vme_user_params p)
{
  vmeSemLock(&VME_ExecParams.semaphore);
  VME_ExecParams.changed = 1;
  VME_ExecParams.events = p.events;
  strcpy(VME_ExecParams.server,p.server);
  if (!VME_ExecParams.running)
    {
      VME_ExecParams.running = 1;
#ifdef vxworks
      sp(vmeExecutor);
#else
      vmeExecutor();
#endif
    }
  vmeSemUnlock(&VME_ExecParams.semaphore);
  return VME_OK;
}

void vmeAddRegister(vme_register* regptr, char* name, char* class, enum vme_regtype type, enum vme_access access, 
		 int offset, vme_vmetype mask, vme_vmetype vmask, int shift, vme_card card)
{
  vmeAddReg(regptr, name, class, type, access, offset, mask, vmask, shift, card);
}
