/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include <stdio.h>
#include "error.h"
#include "low.h"
#include "front.h"

/*
 * vmeSrec2buff below converts Motorola S-Records into arrays of numbers
 *
 */

/************************************************************
 * atoh converts a 2 digit ascii string into a hex number *
 ************************************************************/

static vme_rc atoh(char* a){
  int i;
  for (i=0;i<2;i++) 
    if (strchr("0123456789abcdefABCDEF",a[i]) == NULL)
      return vmeErrorReturn2("Expected hexdecimal number:",VME_BAD_NUMBER);
  return (vme_vmetype)strtol(a,NULL,16);
}
	
/************************************************************************ 
 * stoh converts the first two characters of a string into a hex number *
 ************************************************************************/

static vme_rc stoh(char* s){
  char hex[3];
  if (strlen(s) < 2) vmeErrorReturn2("expected at least two digits",VME_BAD_NUMBER);
  hex[0] = s[0]; hex[1] = s[1]; hex[2] = '\0';
  return atoh(hex);
}

/****************************************************************
 * vmeSrec2buff takes a string containing an S-Record and       *
 *   a pointer to an empty array of vme_vmetype's and returns   *
 *   the number of values taken from the s-record and put into  *
 *   the array.  This will always be either 16 or 0 (zero means *
 *   that the record was the last one in the file).  A return   *
 *   of an vmeError means that an error occured (as usual).     *
 ****************************************************************/

static vme_rc srec2buff(char* rec, vme_vmetype* buff){
  int len,left;
  
  if (debugging) printf("srec2buff\n");
  if (rec[0] == 'S' && rec[1] == '9') return 0; /* end of s-records */
  if (rec[0] != 'S' || rec[1] != '1' || strlen(rec) < 4) return vmeErrorReturn(VME_BAD_SRECORD);
  if (debugging) printf("0...%s\n",rec);
  rec += 2;
  if (debugging) printf("1...%s\n",rec);
  len = stoh(rec);
  rec += 2;
  if (debugging) printf("2...%s\n",rec);
  if (vmeError(len)) return vmeErrorReturn(len);
  if (debugging) printf("Srec len = %d\n",len);
  if (debugging) printf("3...%s\n",rec);
  len -= 3;
  rec += 4;
  if (debugging) printf("Srec len = %d\n",len);
  if (debugging) printf("4...%s\n",rec);
  left = 16-len;
  while (len--) {
    *buff = (vme_vmetype)stoh(rec);
    if (debugging) printf("%d ",*buff);
    if (vmeError(*buff)) return vmeErrorReturn(*buff);
    buff++;
    rec += 2;
  }
  while (left-- > 0) *buff++ = (vme_vmetype)0x00;
/* is it really neccisary to pad out to a minimum of 16 bytes? -- egott */
  if (debugging) printf("done\n");
  return 16;
}  

static vme_rc scanfile(FILE* file, vme_vmetype* buff, int maxsize){
  int len = 0;
  int r;
  int srecord = 0; /* flag */
  char str[256];
  vme_rc rc;
  if (debugging) printf("scanfile\n");
  while ((r = fscanf(file,"%s",str)) > 0 && len < maxsize){
    if (srecord || (len == 0 && str[0] == 'S')) {
      srecord = 1;
      if (debugging) printf("srecord\n");
      rc = srec2buff(str,&buff[len]);
     if (vmeError(rc)) return vmeErrorReturn(rc);
      len += rc;
    } else {
     buff[len++] = (vme_vmetype)strtol(str,NULL,0);
      if (debugging) printf("scanfile: item = %s, len = %d, buff[len-1] = %d\n",str,len,buff[len-1]);
    }
  }
  if (r != EOF) return vmeErrorReturn(VME_CANT_READ_FILE);
  return len;
}  

vme_rc vmeFileWrite(int c, vme_register reg,int addr, char* fname){
  static char* proc = "vmeFileWrite";
  vme_vmetype *buff;
  vme_rc rc,offset;
  int len;
  FILE* file;
  offset = (vme_rc) (addr / sizeof(vme_vmetype));
  if (debugging) printf("front: vmeFileWrite: reg = %s, file = %s\n",reg->name,fname);
  if (c < 0 || c >= reg->card->bases) return vmeErrorReturn2(proc,VME_CARD_DNE);
  if ((reg->type != VME_FIFO && reg->type != VME_CONTIG) || !(reg->access & VME_W_ACCESS)) 
    return vmeErrorReturn2(proc,VME_NO_BUFFWRITE);
  file = fopen(fname,"r");
  if (debugging) printf("file opened\n");
  if (!file) return vmeErrorReturn2(proc,VME_CANT_OPEN_FILE);
  /* be carful that the fclose below is reached */
  buff = (vme_vmetype*) malloc(reg->size*sizeof(vme_vmetype));
  /* be carful that the free() below is reached */
  if (debugging) printf("got buffer of vme_vmetypes, size = %d\n",reg->size);
  len = rc = scanfile(file,buff,reg->size);
  fclose(file);
  if (debugging) printf("file closed\n");
  if (vmeError(rc)) rc = vmeErrorReturn2(proc,len);
  else {
  if (c < 0 || c >= reg->card->bases) rc = vmeErrorReturn(VME_CARD_DNE);
  else rc = reg->buffwrite(reg->card->base[c],reg,buff,len,offset);
  }
  free(buff);
  if (debugging) printf("freeing buffer\n");
  return rc;
}

/* vmeFileRead returns the number of vme_vmetypes read */
/* reads buffer (memory) into file */
vme_rc vmeFileRead(int c, vme_register reg,int addr,int len, char* fname){
  static char* proc = "vmeFileRead";
  vme_rc rc,offset;
  vme_vmetype v,*buff,*buffbase;
  FILE* file;
  offset = (vme_rc) (addr / sizeof(vme_vmetype));
  if (c < 0 || c >= reg->card->bases) return vmeErrorReturn2(proc,VME_CARD_DNE);
  if ((reg->type != VME_FIFO && reg->type != VME_CONTIG) || !(reg->access & VME_R_ACCESS)) 
    return vmeErrorReturn2(proc,VME_NO_BUFFREAD);
  file = fopen(fname,"w");
  /* be sure to close file */
  if (!file) return vmeErrorReturn2(proc,VME_CANT_OPEN_FILE);
  buffbase = buff = (vme_vmetype*) malloc(reg->size*sizeof(vme_vmetype));
  /* be carful that the free() below is reached */
  if (debugging) printf("vmeFileRead\n");
  if (c < 0 || c >= reg->card->bases) rc =  vmeErrorReturn(VME_CARD_DNE);
  else rc =  reg->buffread(reg->card->base[c],reg,buff,len,offset);
  if (!vmeError(rc)){
    len = rc;
    if (debugging) printf("got buffer of size = %d\n",len);
    while (len && fprintf(file,"%d\n",*buff++) > 0) len--;
    if (len) rc = vmeErrorReturn2(proc,VME_CANT_WRITE_FILE);
  }
  if (debugging) printf("closing %s\n",fname);
  fclose(file);
  if (debugging) printf("freeing buffbase\n");
  free(buffbase);
  if (debugging) printf("done\n");
  return rc;
}
