/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include <stdio.h>
#include "semaphore.h"
#include "error.h"
#include "lock.h"

/*
 * reentrant code to restrict access to the "teststand"
 */

/* for the purposes of this demo I'm going to compleatly hide the
 * semaphores....esentialy each server will either have or not
 * have access to the "teststand" untill it gives it up or is granted
 * it respectivly
 */

/*
 * Probably it would be a good idea to alway assume that tasks will exit
 * cleanly (freeing up their resources)....if something dies hard it may
 * be better to continue to withold any resources it has a death grip on.
 */

#ifdef vxworks

struct resource {
  char* name;
  SEM_ID semid;
  int owner;
  int count;
};

static struct resource rlist[]  = { 
  "libLock", NULL, 0, 0,
  "teststand", NULL, 0, 0,
  NULL,NULL,0,0
};

static const int resources = 1; /* number of resources other than libLock */

/* lockup(): Locks up this library.
 *
 * create semaphore for this library if neccisary.
 * use semaphore to lock other tasks out while we do our work.
 * wait on semaphore for up to 120 ticks == 2 seconds. 
 *
 */

static void lockup(){
  vmeSemLock(&rlist[0].semid);
}

static void unlockup(){
  vmeSemUnlock(&rlist[0].semid);
}

static void dump(int res, int id) {
  if ( 0 && res==1 ) 
    printf("resource = (name) %s, (sem) %8d, (id) %5d, (count) %2d, and id = %5d\n",
	   rlist[res].name,rlist[res].semid,rlist[res].owner,rlist[res].count,id);
}

/* vmeLockResource0 and vmeUnlockResource0 do the real work */

/* currently these take a sem each time called which doesn't allow for
 * multiple servers/proccesses with legitamate access to the same resources.
 * This isn't really neccisary and should be removed.
 *
 */

static vme_rc vmeLockResource0(int res,int id){
  if (res < 1 || res > resources) return vmeErrorReturn(VME_RESOURCE_DNE); 
  if (!rlist[res].semid) {
    rlist[res].semid = semMCreate(SEM_Q_FIFO); /* option doesn't really matter at this point */
    if (!rlist[res].semid) return vmeErrorReturn(VME_COULDNT_MAKE_SEM);
  }
  if (rlist[res].owner) {
    if (rlist[res].owner != id) return vmeErrorReturn(VME_RESOURCE_WRONG_ID);
  }
  else rlist[res].owner = id;
  if (semTake(rlist[res].semid,NO_WAIT) == ERROR) return vmeErrorReturn(VME_RESOURCE_NOT_AVAILABLE);
  rlist[res].count++;
  return VME_OK;
}

static vme_rc vmeUnlockResource0(int res,int id){
  if (res < 1 || res > resources) return vmeErrorReturn(VME_RESOURCE_DNE); 
  if (!rlist[res].semid) return vmeErrorReturn(VME_SEMAPHORE_DNE);
  if (!rlist[res].owner) return vmeErrorReturn(VME_RESOURCE_NOT_LOCKED);
  if (rlist[res].owner != id) return vmeErrorReturn(VME_RESOURCE_WRONG_ID);
  if (semGive(rlist[res].semid) == ERROR ) return vmeErrorReturn(VME_ILLEGAL_SEMGIVE);
  if (--rlist[res].count == 0) rlist[res].owner = 0;
  return VME_OK;
}

vme_rc vmeLockResource(enum vme_resource resource,int id){
  vme_rc e;
  if (debugging) printf("vmeLockResource: resource = %d; id = %d\n",resource,id);
  lockup();
  dump(resource,id);
  e = vmeLockResource0(resource,id);
  dump(resource,id);
  unlockup();
  return e;
}

vme_rc vmeUnlockResource(enum vme_resource resource,int id){
  vme_rc e;
  if (debugging) printf("vmeUnlockResource: resource = %d; id = %d\n",resource,id);
  lockup();
  dump(resource,id);
  e = vmeUnlockResource0(resource,id);
  dump(resource,id);
  unlockup();
  return e;
}

#else /* not vxworks ... ie: unix */

/* single threaded == much simpler in this case 
 * also, I'll ignore the possiblity of multiple resources here.
 *
 */

static int resource_key = 0;
static int locked = 0;

vme_rc vmeLockResource(enum vme_resource resource,int id){
  if (debugging) printf("vmeLockResource: resource = %d; id = %d\n",resource,id);
  if (locked && id != resource_key) return vmeErrorReturn(VME_RESOURCE_WRONG_ID);
  if (!locked) resource_key = id;
  locked++;
  return VME_OK;
}

vme_rc vmeUnlockResource(enum vme_resource resource,int id){
  if (debugging) printf("vmeUnlockResource: resource = %d; id = %d\n",resource,id);
  if (!locked) return vmeErrorReturn(VME_RESOURCE_NOT_LOCKED);
  if (id != resource_key) return vmeErrorReturn(VME_RESOURCE_WRONG_ID);
  if (!(--locked)) resource_key = 0;
  return VME_OK; 
}

#endif /* vxworks */

