/* 
 * VME Teststand Software
 *
 * COde revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

/* $Id: low.c,v 1.1.1.1 1995/05/05 16:16:18 egott Exp $ */

#include <stdio.h>
#include <sys/types.h>
#ifdef vxworks
#include <types/vxTypesOld.h>
#endif
#include "error.h"
#include "low.h"
#include "reglist.h"

/* low.c
 *
 * The routines defined here are used as methods in reglist.c and are the code
 * that actually does VME I/O.
 *
 * 
 * In general, functions with "write" in their name put ones where they want to and zeros 
 * where they don't care.  Functions with set in their name maintain state...they read the 
 * register, manipulate the bits they are interested in, and then put the result back.
 * Test and Read functions just read from the given register.
 *
 */


/* 
 * The following three local functions are used for debugging purposes.
 * They take a string which is a taken to be the title of the calling routine, the
 * base of the card to be accessed and the register object to access.  Execpt for read_debug,
 * they also take a value to be written.  They return with the contents of the register
 * in question.  They are meant to be used in such a way that the register is only read from
 * once.  This is particularly important in the case of fifo's where reading from the register
 * more than once would be inapropriat.
 *
 * The information these routines print out is relitivly generic: base address, register offset,
 * absolute address and depending on the routine: the registers value before any operations and
 * the value to be written.
 *
 */
static vme_vmetype write_debug(char* title, vme_vmetype* base, vme_register reg, vme_vmetype value) {
  vme_vmetype xxx;
  xxx = base[reg->offset];
  printf("%s: 0x%8.8x[%2.2d] (ie: 0x%8.8x) = 0x%4.4x <- 0x%4.4x\n",
	 title,base,reg->offset,&base[reg->offset],xxx,value);
  return xxx;
}

static vme_vmetype read_debug(char* title, vme_vmetype* base, vme_register reg) {
  vme_vmetype xxx;
  xxx = base[reg->offset];
  printf("%s: 0x%8.8x[%2.2d] (ie: 0x%8.8x) = 0x%4.4x\n",
	 title,base,reg->offset,&base[reg->offset],xxx);
  return xxx;
}

static void plain_debug(char* title, vme_vmetype* base, vme_register reg, vme_vmetype value) {
  printf("%s: 0x%8.8x[%2.2d] (ie: 0x%8.8x) <- 0x%4.4x\n",
	 title,base,reg->offset,&base[reg->offset],value);
}

/* The definitions of the low level methods begin here.  On the front end they do their respective tasks; on the
 * back end they call a routine which should via the rpc interface ultimatly result in the same method being called
 * in the front end.  All of them, without the debugging code and the difference between the front end version and the
 * back end version are essentialy one line routines.  
 *
 * The global varialbe debugging is defined in error.h/error.c and should be modified via the vmeDebug() command 
 * which is also defined there.  If Debugging is true these routines should print a short debugging message 
 * (see the above routines) to the front end console.
 *
 * The definitions for the vme_register (and vme_reg) type are in reglist.h.  Their values are set in reglist.c
 *
 * All of these routines accept a first parameter "vme_vmetype* base".  This is pointer to the base address
 * of the card to be written to.
 *
 * They also all take a second parameter "vme_register reg".  This is a pointer to a "virtual" register object
 * or structer as defined in reglist.h and reglist.c.  These are the very same objects/structures which will
 * have these procedures as methods, so this pointer is a kind of self pointer.  Several fields from these
 * structuers are used including:
 *   offset -- the offset from the base to the register
 *   mask   -- a bit mask associated with the "virtual" register
 *   shift  -- a bit offset into the register
 *
 * some of the routines take a "vme_vmetype value" which is a value to be written to the "virtual" register.
 *
 */

/* vmeLowSet sets a single bit in a word. */
vme_rc vmeLowSet(vme_vmetype* base, vme_register reg){
#ifdef FRONTEND
  if (debugging) 
    base[reg->offset] = write_debug("vmeLowSet",base,reg,reg->mask) | reg->mask;
  else
    base[reg->offset] |= reg->mask;
  return VME_OK;
#else
  return vmeSet(vmeBase2Index(base),*reg);
#endif
}

vme_rc vmeLowVSet(vme_vmetype* base, vme_register reg){
#ifdef FRONTEND
  if (debugging) 
    base[reg->offset] = (write_debug("vmeLowVSet",base,reg,reg->mask) & reg->vmask) | reg->mask;
  else
    base[reg->offset] = (base[reg->offset] & reg->vmask) | reg->mask;
  return VME_OK;
#else
  return vmeSet(vmeBase2Index(base),*reg);
#endif
}

/* vmeLowClear clears a single bit in a word. */
vme_rc vmeLowClear(vme_vmetype* base, vme_register reg){
#ifdef FRONTEND
  if (debugging) 
    base[reg->offset] = write_debug("vmeLowClear",base,reg,~reg->mask) & ~reg->mask;
  else  
    base[reg->offset] &= ~reg->mask;
  return VME_OK;
#else
  return vmeClear(vmeBase2Index(base),*reg);
#endif
}

vme_rc vmeLowVClear(vme_vmetype* base, vme_register reg){
#ifdef FRONTEND
  if (debugging) 
    base[reg->offset] = (write_debug("vmeLowVClear",base,reg,~reg->mask) & reg->vmask) & ~reg->mask;
  else  
    base[reg->offset] = (base[reg->offset] & reg->vmask) & ~reg->mask;
  return VME_OK;
#else
  return vmeClear(vmeBase2Index(base),*reg);
#endif
}

/* vmeLowbitSet is a write method for single bits.  If value is zero it clears the bit.  Other wise it sets it. */
vme_rc vmeLowBitSet(vme_vmetype* base, vme_register reg, vme_vmetype value){
#ifdef FRONTEND
  return value ? vmeLowSet(base,reg) : vmeLowClear(base,reg);
#else
  return vmeWrite(vmeBase2Index(base),*reg,value);
#endif
}

/* vmeLowTest returns zero if the bit is clear, one if it is not. */
vme_rc vmeLowTest(vme_vmetype* base, vme_register reg){
#ifdef FRONTEND
  if (debugging)
    return ((read_debug("vmeLowTest",base,reg) & reg->mask) != 0);
  else
    return ((base[reg->offset] & reg->mask) != 0);
#else
  return vmeTest(vmeBase2Index(base),*reg);
#endif
}

/* vmeLowNotTest returns one if the bit is clear, zero if it is not. */
/* it is for bits that have "inverted" logic from an earlier design or
 * from intuition or lastly from how we want to use them 
 */

vme_rc vmeLowNotTest(vme_vmetype* base, vme_register reg)
{
#ifdef FRONTEND
  if (debugging)
    return ((read_debug("vmeLowNotTest",base,reg) & reg->mask) == 0);
  else
    return ((base[reg->offset] & reg->mask) == 0);
#else
  return vmeTest(vmeBase2Index(base),*reg);
#endif
}

/* vmeLowBitsSet modifies a group of bits without affecting the surounding bits */
vme_rc vmeLowBitsSet(vme_vmetype* base, vme_register reg, vme_vmetype value){
#ifdef FRONTEND
  if (debugging)
    base[reg->offset] = (write_debug("vmeLowBitsSet",base,reg,value) & (~reg->mask)) | ((value << reg->shift) & reg->mask);
  else
    base[reg->offset] = (base[reg->offset] & (~reg->mask)) | ((value << reg->shift) & reg->mask);
  return VME_OK;
#else
  return vmeWrite(vmeBase2Index(base),*reg,value);
#endif
}

vme_rc vmeLowBitsVSet(vme_vmetype* base, vme_register reg, vme_vmetype value){
#ifdef FRONTEND
  if (debugging)
    base[reg->offset] = ((write_debug("vmeLowBitsVSet",base,reg,value) & reg->vmask) & (~reg->mask)) | ((value << reg->shift) & reg->mask);
  else
    base[reg->offset] = ((base[reg->offset] & reg->vmask) & (~reg->mask)) | ((value << reg->shift) & reg->mask);
  return VME_OK;
#else
  return vmeWrite(vmeBase2Index(base),*reg,value);
#endif
}

/* vmeLowBitsWrite modifies a group of bits and writes zero's to the surounding bits */
vme_rc vmeLowBitsWrite(vme_vmetype* base, vme_register reg, vme_vmetype value){
#ifdef FRONTEND
  if (debugging) 
    plain_debug("vmeLowBitsWrite",base,reg,value);
  base[reg->offset] = (value << reg->shift) & reg->mask;
  return VME_OK;
#else
  return vmeWrite(vmeBase2Index(base),*reg,value);
#endif
}

/* vmeLowBitsRead reads from a group of bits */
vme_rc vmeLowBitsRead(vme_vmetype* base, vme_register reg) {
#ifdef FRONTEND
  if (debugging)
    return((read_debug("vmeLowBitsRead",base,reg) & reg->mask) >> reg->shift);
  else
    return((base[reg->offset] & reg->mask) >> reg->shift);
#else
  return vmeRead(vmeBase2Index(base),*reg);
#endif
}

/* this local function, log2, is just a quick hack implementation of a base 2 log for vmeLowDecodeRead 
 * the same applies to exp2 and vmeLowDecodeWrite
 */
#ifdef FRONTEND
static vme_vmetype log2(vme_vmetype x){
  switch (x) {
  case   1 : return 0;
  case   2 : return 1;
  case   4 : return 2;
  case   8 : return 3;
  case  16 : return 4;
  case  32 : return 5;
  case  64 : return 6;
  case 128 : return 7;
  case 256 : return 8;
  default : printf("x = %d\n",x); return vmeErrorReturn(VME_BAD_CODE);
  }
}

static vme_vmetype exp2(vme_vmetype x){
  return(1 << x);
}
#endif

/* vmeLowDecodeWrite is for writing to groups of mutualy exclusive bits */
vme_rc vmeLowDecodeWrite(vme_vmetype* base, vme_register reg, vme_vmetype value){
#ifdef FRONTEND
  if (debugging)
    plain_debug("vmeLowDecodeWrite",base,reg,value);
  base[reg->offset] = (((1 << value) << reg->shift) & reg->mask);
  return VME_OK;
#else
  return vmeWrite(vmeBase2Index(base),*reg);
#endif
}

vme_rc vmeLowDecodeBitsSet(vme_vmetype* base, vme_register reg, vme_vmetype value){
#ifdef FRONTEND
  if (debugging)
    plain_debug("vmeLowDecodeBitsSet",base,reg,value);
  base[reg->offset] = (base[reg->offset] & (~reg->mask)) | (((1 << value) << reg->shift) & reg->mask);
  return VME_OK;
#else
  return vmeWrite(vmeBase2Index(base),*reg);
#endif
}

vme_rc vmeLowDecodeBitsVSet(vme_vmetype* base, vme_register reg, vme_vmetype value){
#ifdef FRONTEND
  if (debugging)
    plain_debug("vmeLowDecodeBitsVSet",base,reg,value);
  base[reg->offset] = ((base[reg->offset] & reg->vmask) & (~reg->mask)) | (((1 << value) << reg->shift) & reg->mask);
  return VME_OK;
#else
  return vmeWrite(vmeBase2Index(base),*reg);
#endif
}

/* vmeLowDecodeRead is for reading from groups of mutually exclusive bits */
vme_rc vmeLowDecodeRead(vme_vmetype* base, vme_register reg) {
#ifdef FRONTEND
  if (debugging) printf("vmeLowDecodeRead: reg = %s, base = 0x%x, offset = %d, shift = %d, mask = 0x%x\n",reg->name,(int)base,reg->offset,reg->shift,reg->mask);
  return log2((base[reg->offset] & reg->mask) >> reg->shift);
#else
  return vmeRead(vmeBase2Index(base),*reg);
#endif
}

/* vmeLowWrite writes a word */
vme_rc vmeLowWrite(vme_vmetype* base, vme_register reg, vme_vmetype value){
#ifdef FRONTEND
  if (debugging)
    plain_debug("vmeLowWrite",base,reg,value);
  base[reg->offset] = value & reg->mask; /*base is in bytes, offset in words(len of vme_vmetype)*/
  return VME_OK;
#else
  return vmeWrite(vmeBase2Index(base),*reg,value);
#endif
}

/* vmeLowRead reads a word */
vme_rc vmeLowRead(vme_vmetype* base, vme_register reg){
#ifdef FRONTEND
  if (debugging) 
    return(read_debug("vmeLowRead",base,reg) & reg->mask);
  else 
    return(base[reg->offset] & reg->mask);
#else
  return vmeRead(vmeBase2Index(base),*reg);
#endif
}

/* this is the standard low level routine for writting to a fifo from an array/buffer */

vme_rc vmeLowBuffWrite(vme_vmetype* base, vme_register fifo, vme_vmetype* buff, int len, vme_rc m){
#ifdef FRONTEND
  vme_register fullflag;

  fullflag = fifo->fullreg;   /* use this to find out when the fifo is full */
  while (len--){              
    if (fullflag->test(base,fullflag)) return vmeErrorReturn(VME_FIFO_FULL);
    fifo->write(base,fifo,*buff++);                   /* normal write */
    if (vmeErrorState() != VME_OK) return VME_ERROR;  /* check for any errors */
  }
  return VME_OK;
#else
  return vmeBuffWrite(vmeBase2Index(base),fifo,buff,len);
#endif
}

/* this is the standard low level routine for reading from a buffer to an array/buffer */

vme_rc vmeLowBuffRead(vme_vmetype* base, vme_register fifo, vme_vmetype* buff, int len, vme_rc m){
#ifdef FRONTEND
  vme_register emptyflag;
  vme_rc n;

  emptyflag = fifo->emptyreg;            /* use this to find out when fifo is empty */
  for (n=0; n<len || len == 0; n++){     /* len = 0 --> read till fifo is empty */
    if (emptyflag->test(base,emptyflag)) break;               /* fifo is empty */
    buff[n] = fifo->read(base,fifo);                          /* normal read */
    if (vmeErrorState() != VME_OK) return VME_ERROR;          /* make sure no errors */
  }
  if (len != 0 && n != len) vmeErrorReturn(VME_FIFO_EMPTY);   /* if we were forced to stop */
  return n;
#else
  return vmeBuffRead(vmeBase2Index(base),fifo,buff,len);
#endif
}

vme_rc vmeLowContig2Buff(vme_vmetype* base, vme_register array, vme_vmetype* buff, int len, vme_rc m){
  vme_rc n;                              /* items read */
  vme_vmetype count;                     /* current size of array */
  vme_register counterreg,disablereg;      /* registers of interest */

  counterreg = array->counterreg;            /* shorter names are easier */
  disablereg = array->disablereg;        /* don't use this for now ... user is responsable */
  
  if (counterreg == NULL)
    count = array->size;                 /* if countreg DNE; use full size */
  else
    count = counterreg->read(base,counterreg);
  if ((len+m) > array->size || (len+m) > count)  /* make sure we don't try and read too much */
    return vmeErrorReturn(VME_ARRAY_OVERREAD);
  for (n=0; n<len || (len == 0 && (count != 0 && (n+m) < count)); n++){
    buff[n] = base[array->offset+n+m];     /* get the data */
    if (vmeErrorState() != VME_OK) return VME_ERROR; /* ex: bus error */
  }
  return n;  /* return the amount of data read */
}
    
vme_rc vmeLowBuff2Contig(vme_vmetype* base, vme_register array, vme_vmetype* buff, int len, vme_rc m){
  vme_rc n;                              /* items read */
        /* counter and disable are not yet implemented */

  if ((len+m) > array->size )  /* make sure we don't try and write too much */
    return vmeErrorReturn(VME_ARRAY_OVERWRITE);
  for (n=0; n<len ; n++){
    base[array->offset+n+m] =  buff[n] ;     /* write the data */
    if (vmeErrorState() != VME_OK)  return VME_ERROR; /* ex: bus error */
  }
  return n;  /* return the amount of data written */
}
    
