/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karecher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

#include <stdio.h>
#include <sys/types.h>
#ifdef vxworks
#include <types/vxTypesOld.h>
#endif
#include "low.h"
#include "reglist.h"
#include "cardlist.h"
#include "error.h"
#include "unmcp.h"
#include "unmpre.h"
#include "unm604.h"
#include "user.h"

/* reglist.c
 *   all of the virtual registers are defined and initialized here.
 *   To add a new virtual register or to modify othe name of one make sure
 *   to change the list below; the extern list in reglist.h; and add or modify the
 *   init statment in vmeInitRegs.
 */


/* this is the master list of virtural registers */
vme_reglist VME_Reg;

/* these are essentialy stub routines.  They are used as default methods which
 * just return an error rather than doing anything.  They are attached
 * to methods which do not make sense for a given register.  Such as the
 * set method for a full word virtual register
 */
static vme_rc nowrite(vme_vmetype* base, vme_register reg, vme_vmetype value){
  return vmeErrorReturn(VME_NO_WRITE);
}

static vme_rc noread(vme_vmetype* base, vme_register reg){
  return vmeErrorReturn(VME_NO_READ);
}

static vme_rc noset(vme_vmetype* base, vme_register reg){
  return vmeErrorReturn(VME_NO_SET);
}

static vme_rc noclear(vme_vmetype* base, vme_register reg){
  return vmeErrorReturn(VME_NO_CLEAR);
}

static vme_rc notest(vme_vmetype* base, vme_register reg){
  return vmeErrorReturn(VME_NO_TEST);
}

vme_rc nobitswrite(vme_vmetype* base, vme_register reg, vme_vmetype value){
  return vmeErrorReturn(VME_NO_BITSWRITE);
}

vme_rc nobitsread (vme_vmetype* base, vme_register reg){
  return vmeErrorReturn(VME_NO_BITSREAD);
}

vme_rc nobuffwrite(vme_vmetype* base, vme_register reg, vme_vmetype* buff, int len, vme_rc offset){
  return vmeErrorReturn(VME_NO_BUFFWRITE);
}

vme_rc nobuffread(vme_vmetype* base, vme_register reg, vme_vmetype* buff, int len, vme_rc m){
  return vmeErrorReturn(VME_NO_BUFFREAD);
}

/* init is probably the only routine of any significance here.
 *
 * it initializes a register, making choices regarding which low level (see low.c/low.h) routines
 * to use for methods based on the passed parameters.  These choices can always be over written.
 *
 * init gets called in vmeInitRegs.
 */

void vmeAddReg(vme_register* regptr,char* name, char* class, enum vme_regtype type, enum vme_access access, 
		 int offset, vme_vmetype mask, vme_vmetype vmask, int shift, vme_card card)
{
  vme_register reg;

  *regptr = reg = (vme_register) malloc(sizeof(vme_reg));

  reg->name = (char*) malloc(strlen(name) + 1);
  strcpy(reg->name,name);

  reg->class = (char*) malloc(strlen(class) + 1);
  strcpy(reg->class,class);

  reg->type = type;
  reg->access = access;
  reg->card = card;
  reg->index = VME_Reg.number;
  reg->offset = offset;
  reg->shift = shift;
  reg->mask = mask;
  reg->vmask = vmask;
  reg->emptyreg = NULL;
  reg->fullreg = NULL;
  reg->counterreg = NULL;
  reg->disablereg = NULL;
  reg->size = 0;
  reg->write = nowrite;
  reg->read = noread;
  reg->set = noset;
  reg->clear = noclear;
  reg->test = notest;
  reg->buffwrite = nobuffwrite;
  reg->buffread = nobuffread;
  
  switch (type) {
  case VME_BIT : 
    if (access & VME_R_ACCESS) {
      reg->read = vmeLowTest;
      reg->test = vmeLowTest;
    }
    if (access & VME_W_ACCESS) {
      reg->write = vmeLowBitSet;
      if (vmask)
	reg->set = vmeLowVSet;
      else
	reg->set = vmeLowSet;
      if (!(access & VME_V_ACCESS)) {
	if (vmask)
	  reg->clear = vmeLowVClear;
	else
	  reg->clear = vmeLowClear;
      }
    }
    break;
  case VME_BITS :
    if (access & VME_R_ACCESS) {
      if (access & VME_D_ACCESS) 
	reg->read = vmeLowDecodeRead;
      else 
	reg->read = vmeLowBitsRead;
    }
    if (access & VME_W_ACCESS){
      if (access & VME_D_ACCESS){
	if (vmask)
	  reg->write = vmeLowDecodeBitsSet;
	else
	  reg->write = vmeLowDecodeBitsSet;
      } else {
	if (vmask)
	  reg->write = vmeLowBitsSet;
	else
	  reg->write = vmeLowBitsSet;
      }
    }
    break;
  case VME_WORD :
    if (access & VME_R_ACCESS) reg->read = vmeLowRead;
    if (access & VME_W_ACCESS) reg->write = vmeLowWrite;
    break;
  case VME_CONTIG :
    if (access & VME_R_ACCESS) reg->read = vmeLowRead;
    if (access & VME_W_ACCESS) reg->write = vmeLowWrite;
    break;
  }

  VME_Reg.list[VME_Reg.number++] = reg;
}

/* fifo's objects have some extra info in them...fifoinit is usualy called on
 * a fifo register just after init is in vmeInitRegs
 */
void vmeAddFifo(vme_register fifo, vme_writefunc wf, vme_readfunc rf, 
		     vme_buffwritefunc bwf, vme_buffreadfunc brf, 
		     vme_register emptyreg, vme_register fullreg, int size)
{
  if (wf) fifo->write = wf;
  if (rf) fifo->read = rf;
  if (bwf) fifo->buffwrite = bwf;
  if (brf) fifo->buffread = brf;
  fifo->emptyreg = emptyreg;
  fifo->fullreg = fullreg;
  fifo->size = size;
}

void vmeAddContig(vme_register array, 
		       vme_buffwritefunc bwf, vme_buffreadfunc brf,
		       vme_register counterreg, vme_register disablereg, int size)
{
  if (bwf) array->buffwrite = bwf;
  if (brf) array->buffread = brf;
  array->counterreg = counterreg;
  array->disablereg = disablereg;
  array->size = size;
}

static void deleteReg(vme_register reg)
{
  free(reg->name);
  free(reg->class);
  free(reg);
}

/* 
   if a clean way to shut down the server ever gets provided, it should
   call vmeDeleteCards() and vmeDeleteRegs().
*/

void vmeDeleteRegs()
{
  static int first_time = 1;  /* to make sure VME_Reg.number gets initialized to zero */

  if (first_time)
    {
      VME_Reg.number = 0;
      first_time = 0;
    }
  else
    {
      while(VME_Reg.number)
	{
	  deleteReg(VME_Reg.list[--VME_Reg.number]);
	}
    }
}

vme_rc vmeInitRegs()
{
  vmeDeleteRegs();

  vmeAddUserRegisters();

  return VME_OK;
}

/* given a vme_regtype, returns an apropriate string */
char *vmeRegTypeString(enum vme_regtype rt){
  switch(rt){
  case VME_BIT : return "bit";
  case VME_BITS : return "bits";
  case VME_WORD : return "word";
  case VME_FIFO : return "fifo";
  case VME_CONTIG : return "contig";
  case VME_OTHER : return "other";
  default : return "unknown";
  }
}



