/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 */

/* reglist.h
 *
 * (c) 1994 by Eric Gottlieb
 * for the VME II Test Stand Control Software
 *
 */

#ifndef reglist_h
#define reglist_h

#include "error.h"
#include <sys/types.h>
#ifdef vxworks
#include <types/vxTypesOld.h>
#endif

/* reglist comes from "list of registers"
 * registers in this context are more generalized than normal and
 *   refer to anthing from a single bit, to a group of bits up to 
 *   a whole word (unsigned in our case).
 *
 * In order to distinguish this sense of register from the normal one
 * we will attempt to use the term "virtual" register for these and
 * "physical" register for actual words accessible via the VME bus.
 *
 * registers are represented by a "struct" which contains information
 *   about them including the methods to be used to manipulate them.
 *   This amounts to being an object (as in object oriented) and
 *   virtual registers will be refered to both as being objects and/or
 *   structs
 *
 */

/* regnode methods:
 *
 * all of these functions take at least two arguments: a base address
 * and a pointer to a register (a "self" pointer).  Generic information
 * (such as the register offset) can be gotten from the register structure 
 * pointer.  The base address is neccisary since there may be multiple
 * instances of the same register on different copies of the same card
 * (multiple fib cards for instance).
 * 
 * *****************************************************************
 * ********* important note **************************************** 
 *
 * currently we are accessing the VME bus in D16 mode (16 bit words)
 * vme_vmetype is acordingly a vme_vmetype or a unsigned short integer
 * (which happens to be 16 bits long on 680x0 proccessors such as
 * is in the mvme162 card which is our front end computer and which does
 * our VME accesses).
 *
 * Offsets then are NOT in bytes but rather in 16 bit (or 2 byte) words.
 * Future systems my use D32 mode to access the VME bus.  In that case
 * the offsets will be in 32 bit (or 4 byte) words.
 *
 * using an offset that is the length of the data type being accessed 
 * has the advantage that it makes pointer arithmatic simpler and 
 * that changes such as going from D16 to D32 mode VME accesses will
 * not require changing the offsets of the virtual registers.
 *
 * the base is a (vme_vmetype *).  this means that the offsets are not in bytes, 
 * but rather in 16bit words.  This is done to acomadate the D16 mode that 
 * all cards will be accessed in and to keep pointer arithmatic simple.
 *
 * *****************************************************************
 * *****************************************************************
 * */

/* this is the size of the data that can be read from or written to
 * the cards via the VME bus.  It is typedef'ed so that a transition
 * to D32 (32 bit) from D16 (16 bit) will be as simple (hopefully) as
 * changing this one line of code.  Is D32 now!! */

typedef unsigned long int vme_vmetype;

/* the real definition of struct vme_reg is below...this is neccisary for 
 * some typedefs which are used within it.
 */

struct vme_reg; 
typedef struct vme_reg* vme_register; /* a useful typedef */

/* these are the prototypes for the method functions The functions
   themselfs are defined in low.c and low.h */

typedef vme_rc (* vme_writefunc)     (vme_vmetype* base, vme_register reg, vme_vmetype value);
typedef vme_rc (* vme_readfunc)      (vme_vmetype* base, vme_register reg);
typedef vme_rc (* vme_setfunc)       (vme_vmetype* base, vme_register reg);
typedef vme_rc (* vme_buffwritefunc) (vme_vmetype*, vme_register reg, vme_vmetype* buff, int len, vme_rc offset);
typedef vme_rc (* vme_buffreadfunc)  (vme_vmetype*, vme_register reg, vme_vmetype* buff, int len, vme_rc offset);

/* used to describe cards and registers 
 * VME_UNKNOWN is provided for error reporting
 * VME_GENERIC is here just for flexability....it is not currently used anywhere in the code.
 *
 * VME_OTHER is similarly provided mostly for error reporting and/or generality.
 */

enum vme_regtype {VME_OTHER, VME_BIT, VME_BITS, VME_WORD, VME_FIFO, VME_CONTIG};

/* this defines the varios ways in which a register can be accessed 
 * For example some registers are read only, so they would have their
 * access field logicaly OR'ed w/ VME_R_ACCESS, but not w/ VME_W_ACCESS.
 *
 */

enum vme_access {VME_NOACCESS = 0x00, VME_R_ACCESS = 0x01, VME_W_ACCESS = 0x02, 
		 VME_RW_ACCESS = 0x03, VME_V_ACCESS = 0x04, VME_WV_ACCESS = 0x06,
		 VME_D_ACCESS = 0x08, VME_RD_ACCESS = 0x09, VME_S_ACCESS = 0x10};

/* W == WRITE, R == READ, V == volatile, D == Decode, S == SPECIAL */
/* volatile means that it doesn't maintain it's state.  ie: you may not read back
 * what you wrote.
 *
 * decode means to decode a bit field (ie: 1000 -> 4 not 8)
 *
 */

/* vme_reg is the structure which contains information about a virtual register.  It has
 * the following fields:
 
 * name -- useful for displaying; used in tcl accesing.  A typical name might be "sfifo" for "silo fifo"
 * class -- with name makes for a two level hirarchy... used in gui creation. 
 * type -- vme_regtype is defined above.  typical value: VME_WORD or VME_FIFO 
 * access -- vme_access is defined above.  typeical value: VME_R_ACCESS | VME_W_ACCESS 
 * card -- vme_card is defined in cardlist.h  A vme_card is a struct like a vme_reg.  It contains a cardtype field
     (cardtype is defined above) which tells what kind of card it is 
 * index -- It turns out to be useful to keep registers in a list or array and for them to know their own index
     into that array.  

 * offset -- this is the number of words (16 or 32 bit depending on the access mode being used 
     (ie: depending on the type of vme_vmetype) from the apropriate card's base address
     to the physical register in question. 
 * shift -- this is the bit offset in the physical register to the group of bits in question.
     shift only makes sense for virtual registers that have more than one bit in them,
     but share a single physical register with other virtual registers.  ie: they have
     a vme_regtype of VME_BITS 

 * size -- the size of the fifo 
 * fullreg -- the virtual register which can be tested to see if the fifo is full 
 * emptyreg -- the virtual register which can be tested to see if the fifo is empty 

 * methods:
 * read and write work on one bit registers as well as on multi bit registers.
 * set clear and test are for accessing bit like registers 
 * buffwrite and buffread are for writting and reading entire arrays or buffers 

 */ 

struct vme_card; /* needed because we can't #include "cardlist.h" here (it includes us instead) */

struct vme_reg {
/* info: these fields contain information about the register */
  char *name;
  char *class;
  enum vme_regtype type;
  enum vme_access access;
  struct vme_card* card;
  int    index;
/* definition: these fields essentialy define the register */
  int    offset;     /* physical register offset into card */
  int    shift;      /* bit offset */
  vme_vmetype mask;  /* masks in relivent bits */
  vme_vmetype vmask; /* masks out voiltile bits */
/* fifo stuff: these fields are useful for fifo's */
  int size;
  struct vme_reg* fullreg;
  struct vme_reg* emptyreg;
  struct vme_reg* counterreg;
  struct vme_reg* disablereg;
/* methods: these are functions which are used to access the virtual register */
  vme_writefunc     write;
  vme_readfunc      read;
  vme_setfunc       set;
  vme_setfunc       clear;
  vme_readfunc      test;
  vme_buffwritefunc buffwrite;
  vme_buffreadfunc  buffread;
};
typedef struct vme_reg vme_reg;

/*define reg_data structure to hand data down to frontend for chip register write*/
struct chip_reg {
   char byte0;
   char byte1;
   char byte2;
   char byte3;
   char byte4;
   char byte5;
   char byte6;
   char byte7;
 };
typedef struct chip_reg chip_reg;
/* lists of registers are always smaller than this */
#define VME_MAX_NUMBER_REGISTERS 512

/* the master list of registers has this form */
struct vme_reglist {
  vme_register list[VME_MAX_NUMBER_REGISTERS];
  int number;
};
typedef struct vme_reglist vme_reglist;

/* VME_Reg is the master list of registers */
extern vme_reglist VME_Reg;

/* the global functions defined in reglist.c */
vme_rc vmeInitRegs();
char *vmeRegTypeString(enum vme_regtype rt);

void vmeDeleteRegisters();

void vmeAddReg(vme_register* regptr, char* name, char* class, enum vme_regtype type, enum vme_access access, 
		 int offset, vme_vmetype mask, vme_vmetype vmask, int shift, struct vme_card* card);

void vmeAddFifo(vme_register fifo, vme_writefunc wf, vme_readfunc rf, 
		     vme_buffwritefunc bwf, vme_buffreadfunc brf, 
		     vme_register emptyreg, vme_register fullreg, int size);

void vmeAddContig(vme_register array, 
		       vme_buffwritefunc bwf, vme_buffreadfunc brf,
		       vme_register counterreg, vme_register disablereg, int size);

#endif

