/* 
 * VME Teststand Software
 *
 * Code revised by Armin Karcher (karcher@design.lbl.gov)
 *
 * Design and Template by:
 *   Eric Gottlieb  (egott@unm.edu)
 *   Tim Thomas     (thomas@hev1.phys.unm.edu)
 *   Micheal Gold   (gold@hepv1.phys.unm.edu)
 *
 *----------------------------------------------------------------
 *  Modified to include rpc support for motor control functions
 *   Sanjay Hortikar (horti@physics.ucsb.edu)
 *-------------------------------------------------------------   
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#ifdef vxworks
#include <types/vxTypesOld.h>
#endif
#include "error.h"
#include "reglist.h"
#include "cardlist.h"
#include "rpc_client.h"
#include "front.h"
#include "unmpre.h"
#include "lock.h"
#include "server_date.h"
#include "rpc.h"
/*
 * _client.c
 * ---------
 *
 * "support" routines are static and local only.
 *
 * "client configuration" routines provide for control of the rpc interface.
 *
 * "remote" routines call procedures in _clnt.c code (which is generated by
 * rpcgen.new):
 *
 * vmeDoIt --> vmedoit_1 -------RPC-----> vmedoit_1_svc --> vmeDoIt
 * backend       _client.c   _clnt.c   _svc.c       _server.c      frontend
 *
 * file.x, file_client.c, and file_server.c must be maintained by hand.
 * file_clnt.c, file_svc.c, file_xdr.c, and file.h come from file.x and rpcgen.new.
 *
 */


/* 
 * support routines and global variables
 *
 * errorcode is used by routines that call init in the case that init returns an error.
 * clnt is needed for RPC's.
 * host is needed for creating a client data structure (clnt).
 *
 * init() is always called by each "remote" routine before any rpc calls.  It takes
 * care of any neccisary one time and/or chronic initializations.
 *
 * passback() takes care of any neccisary cleaning up that needs to be done at the end
 * of a call.
 *
 */

static vme_rc errorcode = VME_OK;
static CLIENT *clnt;
static char host[256] = "";

static vme_rc passback(vme_rpc_return* rc){
  char* errmess;
  if (rc != NULL) {
    if (rc->error != 0) 
      while (errmess = vmeFrontError(rc->error)) 
	vmeErrorReturn1(errmess);
    return rc->value;
  } else {
    clnt_perror(clnt, "call failed:");
    return vmeErrorReturn(VME_RPC_FAILED);
  }
}

static void burn(){
  if (clnt) clnt_destroy(clnt);
  clnt = NULL;
  sprintf(host,"%s",""); 
/* if host is not set to "" then it remains as a default.  If an rpc
 * is attempted, this default will be tried, and if successful the
 * teststand will be locked.  Setting it to "" means that a setserverhost
 * command must be successful before any rpc's can be done. */
}

static vme_rc init(){
  if (*host == 0) return vmeErrorReturn(VME_NO_SERVERHOST);
  if (clnt != NULL) return VME_OK;
  clnt = clnt_create(host, TESTSTAND_PROGRAM, TESTSAND_VERSION, "tcp");
  if (clnt != NULL) return VME_OK;
  clnt_pcreateerror(host);
  burn(); 
  return vmeErrorReturn(VME_NO_CLIENT);
}

/* 
 * client configuration routines
 *
 * these routines choose what machine to look for the teststand server on.
 *
 */

vme_rc vme_UnsetServerHost(){
  vme_rc rc;
  if ((rc = vmeUnlockResource(VME_RES_TESTSTAND,getpid()))) return rc;
  burn();
  return VME_OK;
}

vme_rc vme_SetServerHost(char *hostname){
  vme_rc rc;
  if (*host != 0 && vmeError(rc = vme_UnsetServerHost())) return rc;
  sprintf(host,"%s",hostname);
  return init();
}

/* 
 * remote routines 
 *
 * init sets errorcode (global to file context only (ie: static))
 * passback takes care of converting from return by referance to return by value.
 *
 */

/* vmeResetFront is described in more detail in front.c */
/* 
 * vmeresetfront_1 takes an additional string parameter which is used to
 * verify that the server being used is up to date.  it is cast (char*) since
 * the variable being pased is prototyped as const char* which causes gcc to 
 * generate a warning that we wish to ignore at this point.
 *
 */
vme_rc vmeResetFront(){
  if (init() != VME_OK) return 0;  
  return passback(vmeresetfront_1((char*)VME_ServerCompileDate,clnt));
}

vme_rc vmeLockResource(enum vme_resource resource, int resource_key){
  if (init() != VME_OK) return 0;  
  return passback(vmelockresource_1(resource,resource_key, clnt));
}

vme_rc vmeUnlockResource(enum vme_resource resource, int resource_key){
  if (init() != VME_OK) return 0;  
  return passback(vmeunlockresource_1(resource,resource_key, clnt));
}

vme_rc vmeWrite(int c,vme_register r, vme_vmetype v)
{
  if (init() != VME_OK) return 0;
  return passback(vmewrite_1(c,r->index, v, clnt));
}

vme_rc vmeRead(int c,vme_register r)
{
  if (init() != VME_OK) return 0;
  return passback(vmeread_1(c,r->index, clnt));
}

vme_rc vmeSet(int c,vme_register r)
{
  if (init() != VME_OK) return 0;
  return passback(vmeset_1(c,r->index, clnt));
}

vme_rc vmeClear(int c,vme_register r)
{
  if (init() != VME_OK) return 0;
  return passback(vmeclear_1(c,r->index, clnt));
}

vme_rc vmeTest(int c,vme_register r)
{
  if (init() != VME_OK) return 0;
  return passback(vmetest_1(c,r->index, clnt));
}

vme_rc vmeFileWrite(int c, vme_register r,int add, char* fname){
  if (init() != VME_OK) return 0;
  return passback(vmefilewrite_1(c,r->index,add,fname,clnt));
}

vme_rc vmeFileRead(int c, vme_register r,int add, int len, char* fname){
  if (init() != VME_OK) return 0;
  return passback(vmefileread_1(c,r->index,add,len,fname,clnt));
}

vme_rc vmePmCommand(int c,int opcd,int addr,vme_vmetype chan,vme_vmetype cnt,int dir, vme_vmetype* data)
{
  vme_rpc_chip_reg chip_reg;
  if (init() != VME_OK) return 0;
  chip_reg.error = 0;
  chip_reg.data.data_val = data;
  chip_reg.data.data_len = 32;
  return passback(vmepmcommand_1(c, opcd, addr, chan, cnt, dir, chip_reg, clnt));
}  

vme_rc vmePmCommandAll(int c, int opcd, int nchips, int* addr_array, 
		       int* chan_array, int cnt, int dir, vme_vmetype* data)
{
  int_vme_rpc_buffer addr_out;
  int_vme_rpc_buffer chan_out;
  vme_rpc_buffer     data_out;

  if (init() != VME_OK) return 0;
  addr_out.error = 0;
  chan_out.error = 0;
  data_out.error = 0;

  addr_out.data.data_val = addr_array;
  chan_out.data.data_val = chan_array;
  data_out.data.data_val = data;

  addr_out.data.data_len = nchips;
  chan_out.data.data_len = nchips;
  data_out.data.data_len = 32*nchips;

  return passback(vmepmcommandall_1(c, opcd, nchips, addr_out, chan_out,
				    cnt, dir, data_out, clnt));
}


vme_rc vmeBuffWrite(int c, vme_register r, vme_vmetype* data, int length){
  vme_rpc_buffer buff;
  if (init() != VME_OK) return 0;
  buff.error = 0;
  buff.data.data_val = data;
  buff.data.data_len = length;
  return passback(vmebuffwrite_1(c,r->index,buff,clnt));
}


vme_rc vmeBuffRead(int c, vme_register r, vme_vmetype* data, int length){
  vme_rpc_buffer* buff;
  char* errmess;
  int i;
  vme_rc rc;
  if (init() != VME_OK) return 0;
  buff = vmebuffread_1(c,r->index,length,clnt); /* read real buffer into buff variable on frontend  remember maxbuffsize in rpc.x!!*/
  if (buff->data.data_len > 0)            /* get data across bridge */
    for (i=0;i<buff->data.data_len;i++) data[i] = buff->data.data_val[i];
  if (buff->error != 0)
    while (errmess = vmeFrontError(buff->error))
      vmeErrorReturn1(errmess);
  rc = buff->value;
  xdr_free(xdr_vme_rpc_buffer,buff);
  return rc;
}


vme_rc vmeVmeBuffRead(vme_vmetype* base, int offset, vme_vmetype* data, 
int length) {
  vme_rpc_buffer* buff;
  char* errmess;
  int i;
  vme_rc rc;
  
  if (init() != VME_OK) return 0;
  buff = vmevmebuffread_1( (int) base, offset, length, clnt);
  if (buff->data.data_len > 0)
      for (i=0; i<buff->data.data_len; i++) data[i] = buff->data.data_val[i];
  if (buff->error != 0)
      while (errmess = vmeFrontError(buff->error))
	  vmeErrorReturn1(errmess);
  rc = buff->value;
  xdr_free(xdr_vme_rpc_buffer, buff);
  return rc;
}

/*----------------------------------------------------------------------*/
/* copy of vmeVmeBuffRead that skips one word                           */
/* this is ugly, I could have added a parameter to vmeVmeBuffRead       */
/* but so it goes.....                                                  */
/*----------------------------------------------------------------------*/
vme_rc vmeVmeBuffRead2(vme_vmetype* base, int offset, vme_vmetype* data, 
int length) {
  vme_rpc_buffer* buff;
  char* errmess;
  int i;
  vme_rc rc;
  
  if (init() != VME_OK) return 0;
  buff = vmevmebuffread2_1( (int) base, offset, length, clnt);
  if (buff->data.data_len > 0)
      for (i=0; i<buff->data.data_len; i++) data[i] = buff->data.data_val[i];
  if (buff->error != 0)
      while (errmess = vmeFrontError(buff->error))
	  vmeErrorReturn1(errmess);
  rc = buff->value;
  xdr_free(xdr_vme_rpc_buffer, buff);
  return rc;
}

vme_rc vmeFrontDisplay(char* mess){
  if (init() != VME_OK) return 0;
  return passback(vmefrontdisplay_1(mess,clnt));
}

vme_rc vmeGetState(int c, int classindex, vme_register* registers, vme_vmetype* values){
  vme_rpc_state* state;
  char* errmess;
  int i;
  vme_rc rc;

  if (init() != VME_OK) return 0;
  state = vmegetstate_1(c,classindex,clnt);
  if (state->error != 0)
    while (errmess = vmeFrontError(state->error))
      vmeErrorReturn1(errmess);
  if (vmeErrorState() == 0)
    for (i = 0; i<state->values.values_len; i++) {
      values[i] = state->values.values_val[i];
      registers[i] = VME_Reg.list[state->reglist.reglist_val[i]];
    }
  xdr_free(xdr_vme_rpc_state,state);
  return state->values.values_len;
}
       
vme_vmetype* vmeAddCard(vme_card card, char* instname, vme_vmetype* base)
{
  vme_vmetype* address;

  printf("client: vmeAddCard\n");
  printf("client: index = %d, instname = %s, base = %d\n",card->index, instname, base);
  if (init() != VME_OK) return 0;
  address = (vme_vmetype*)passback(vmeaddcard_1(card->index, instname, (int)base, clnt));
  if (!vmeErrorState()) 
    {
      vmeAddCardBase(card, instname, base);
    }
  return address;
}

void vmeAddCardType(char* name)
{
  vme_card dummy;

  if (init() == VME_OK)
    {
      passback(vmeaddcardtype_1(name,clnt));
      if (!vmeErrorState())
	{
	  vmeAddCardClass(&dummy,name);
	}
    }
}

vme_rc vmeWait(int card, vme_register bit_reg, vme_vmetype pole, int seconds){
  if (vmeError(init())) return 0;
  return passback(vmewait_1(card,bit_reg->index,pole,seconds,clnt));
}

char* vmeFrontError(int pid){
  static char buff[256];
  static char** result;
  result = (char**)&buff;
  if (vmeError(init())) return NULL;
  result = vmefronterror_1(pid,clnt);
  if (strlen(*result) == 0) return NULL;
  strcpy(buff,*result);
  xdr_free(xdr_string,result);
  if (result != NULL) return buff;
  clnt_perror(clnt, "call failed:");
  return NULL;
}

vme_rc vmeVmeWrite(vme_vmetype* base,int offset,vme_vmetype value){
  if (vmeError(init())) return 0;
  return passback(vmevmewrite_1((int)base,offset,(int)value,clnt));
}

vme_rc vmeVmeRead(vme_vmetype* base,int offset){
  if (vmeError(init())) return 0;
  return passback(vmevmeread_1((int)base,offset,clnt));
}

vme_rc vmeDebug(int level){
  if (init() != VME_OK) return 0;  
  return passback(vmedebug_1(level, clnt));
}

vme_rc vmeExecControl(struct vme_user_params p)
{
  if (init() != VME_OK) return 0;  
  return passback(vmeexeccontrol_1(p, clnt));
}

void vmeAddRegister(vme_register* regptr, char* name, char* class, enum vme_regtype type, enum vme_access access, 
		 int offset, vme_vmetype mask, vme_vmetype vmask, int shift, vme_card card)
{
  if (init() == VME_OK)
    { 
      /* take care of this end */
      vmeAddReg(regptr, name, class, type, access, offset, mask, vmask, shift, card);
      
      /* deal with front end */
      passback(vmeaddregister_1(name, class, type, access, offset, mask, vmask, shift, card->index, clnt));
    }
}


/***************************************************
 motor control functions  follow............
*/
#define MBASE     0x800000  
#define A24OFFSET 0xe0000000


int mExec(char * buff){
   int *value;
   value = mexec_1( MBASE+A24OFFSET, buff,clnt);
   return *value;
}

int mStat(int axis){
  int *value;
  value=mstat_1(axis,clnt);
   return *value;
}

#define PORT "/tyCo/1"
#define CR  13
#define LF  10
#define MODE 0
#define BUFF_LEN 100

int oPort(char * data){
  char buff[BUFF_LEN+2];
  int len,i;
  strcpy(buff,data);
  len=strlen(buff);
  buff[len]=CR;
  buff[len+1]=LF;
  buff[len+2]='\0';
  return *oport_1(buff,PORT,MODE,clnt);
}

char *iPort(){
   static char buff[100];
   strcpy(buff,iport_1(PORT,MODE,clnt)->reply);
/*   printf("client:reply is %s\n",buff);  */
   return buff;
 }
/*-------------------------------------------------------------------*/
/* This is the routine called to acquire one event.  See the vxworks */
/* code for the meaning of the parameters (eventseq.c)               */
/*-------------------------------------------------------------------*/
char* EventSeq(int c,      int opcode, int nled, int chamsk, 
               int wrdcnt, int add,    int noop) {
  static char return_message[200];
  md_ret * evseq;
  evseq = eventseq_1(c,opcode,nled,chamsk,wrdcnt,add,noop,clnt);
  strcpy(return_message,evseq->reply);
  xdr_free(xdr_md_ret,evseq);
  return return_message;
}


